// Copyright (C) 2009  Daniel Filkovic

// This file is part of APAME-WCS version 3.0.

// APAME-WCS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME-WCS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME-WCS.  If not, see <http://www.gnu.org/licenses/>.

// APAME-WCS - Aircraft PAnel MEthod-Wing Coefficients Solver
// - Pre and Post-Processor coupled with APAME solver for Simple wing
//   form analysis
//
// Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
//             - Kesic; Osnove aerodinamike
//             - Jankovic; Mehanika leta zrakoplova
//             - Abbott, Doenhoff; Theory of wing sections
//
// file aero_force.sci

function aero_force(AL,CX,CZ,CM,AR)

getf('polyfit.sci')
getf('polyval.sci')

pi = 3.1415926535897932;

Cd = zeros(1,length(AL));
Cl = zeros(1,length(AL));
for i=1:length(AL)
  Cd(i) =  CX(i)*cos(AL(i)) + CZ(i)*sin(AL(i));
  Cl(i) = -CX(i)*sin(AL(i)) + CZ(i)*cos(AL(i));
end

if length(AL) == 1
  disp('Drag coefficient Cd=')
  disp(Cd)
  disp('Lift coefficient Cl=')
  disp(Cl)
  disp('Moment coefficient Cm=')
  disp(CM)
elseif length(AL) > 2
  poly_Cl = polyfit(AL,Cl,1);
  Cl_al = poly_Cl(2);
  Cl_al_0 = poly_Cl(1);
  poly_Cd = polyfit(AL,Cd,2);
  Cd_min = -poly_Cd(2)^2/4/poly_Cd(3)+poly_Cd(1);
  Cl_minCd = polyval(poly_Cl,polyval(poly_Cd,Cd_min));
  poly_e = polyfit(Cl,Cd,2);
  e = 1/AR/pi/poly_e(3);
  poly_Cm = polyfit(AL,CM,1);
  Cm_al = poly_Cm(2);
  Cm_al_0 = poly_Cm(1);
  disp('Lift coefficient slope Cl_alfa=')
  disp(Cl_al)
  disp('Lift coefficient at zero AOA Cl_alfa_0=')
  disp(Cl_al_0)
  disp('Lift coefficient at minimum drag Cl_minCd=')
  disp(Cl_minCd)
  disp('Oswald coefficient e=')
  disp(e)
  disp('Moment coefficient slope Cm_alfa=')
  disp(Cm_al)
  disp('Moment coefficient at zero angle of attack Cm_alfa_0=')
  disp(Cm_al_0)
else
  poly_Cl = polyfit(AL,Cl,1);
  Cl_al = poly_Cl(2);
  Cl_al_0 = poly_Cl(1);
  disp('Lift coefficient slope Cl_alfa=')
  disp(Cl_al)
  disp('Lift coefficient at zero AOA Cl_alfa_0=')
  disp(Cl_al_0)
  disp('Drag coefficient Cd=')
  disp(Cd)
  poly_Cm = polyfit(AL,CM,1);
  Cm_al = poly_Cm(2);
  Cm_al_0 = poly_Cm(1);
  disp('Moment coefficient slope Cm_alfa=')
  disp(Cm_al)
  disp('Moment coefficient at zero angle of attack Cm_alfa_0=')
  disp(Cm_al_0)
end

endfunction
