// Copyright (C) 2009  Daniel Filkovic

// This file is part of APAME-WCS version 3.0.

// APAME-WCS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME-WCS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME-WCS.  If not, see <http://www.gnu.org/licenses/>.

// APAME-WCS - Aircraft PAnel MEthod-Wing Coefficients Solver
// - Pre and Post-Processor coupled with APAME solver for Simple wing
//   form analysis
//
// Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
//             - Kesic; Osnove aerodinamike
//             - Jankovic; Mehanika leta zrakoplova
//             - Abbott, Doenhoff; Theory of wing sections
//
// file meshing.sci

function [x,y,z] = meshing(M1,Mp,pi,M2,N1,span,xp1,zp1,xp2,zp2);

Mp1 = Mp+1;
Np = N/2;
Np1 = Np+1;

ykoef = zeros(M1,1);
for i = 1:Mp
    ykoef(i) = cos(pi/Mp*(i+Mp-1))*0.5-0.5;
    ykoef(M2-i) = -ykoef(i);
end
yrasp = ykoef*(span/2);

x = zeros(N1,Mp1,4);
y = zeros(N1,Mp1,4);
z = zeros(N1,Mp1,4);

// left wing
for i = 1:Mp1
    for j = 1:N1
        x(j,i,1) = xp1(j)*(1-abs(ykoef(i)))+xp2(j)*abs((ykoef(i)));
        z(j,i,1) = zp1(j)*(1-abs(ykoef(i)))+zp2(j)*abs((ykoef(i)));
        y(j,i,1) = yrasp(i);
    end
end
// right wing
for i = 1:Mp1
    for j = 1:N1
        x(j,i,2) = xp1(j)*(1-abs(ykoef(Mp+i)))+xp2(j)*abs((ykoef(Mp+i)));
        z(j,i,2) = zp1(j)*(1-abs(ykoef(Mp+i)))+zp2(j)*abs((ykoef(Mp+i)));
        y(j,i,2) = yrasp(Mp+i);
    end
end
// left wingtip
x(1:Np1,1,3) = x(Np1:-1:1,1,1);
x(1:Np1,2,3) = x(Np1:N1,1,1);
y(1:Np1,1,3) = y(Np1:-1:1,1,1);
y(1:Np1,2,3) = y(Np1:N1,1,1);
z(1:Np1,1,3) = z(Np1:-1:1,1,1);
z(1:Np1,2,3) = z(Np1:N1,1,1);
// right wingtip
x(1:Np1,1,4) = x(Np1:N1,Mp1,2);
x(1:Np1,2,4) = x(Np1:-1:1,Mp1,2);
y(1:Np1,1,4) = y(Np1:N1,Mp1,2);
y(1:Np1,2,4) = y(Np1:-1:1,Mp1,2);
z(1:Np1,1,4) = z(Np1:N1,Mp1,2);
z(1:Np1,2,4) = z(Np1:-1:1,Mp1,2);

endfunction
