// Copyright (C) 2009  Daniel Filkovic

// This file is part of APAME-WCS version 3.0.

// APAME-WCS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME-WCS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME-WCS.  If not, see <http://www.gnu.org/licenses/>.

// APAME-WCS - Aircraft PAnel MEthod-Wing Coefficients Solver
// - Pre and Post-Processor coupled with APAME solver for Simple wing
//   form analysis
//
// Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
//             - Kesic; Osnove aerodinamike
//             - Jankovic; Mehanika leta zrakoplova
//             - Abbott, Doenhoff; Theory of wing sections
//
// file read_results.sci

function [origin,S,CX,CZ,CM,cp,v,doublet,vx,vy,vz,p_stat,p_dyna,p_mano] = read_results(AL,N,M)

N = [N,N,N/2,N/2];
M = [Mp,Mp,1,1];

results_file = file('open','wing.res','old');

komp = 0;
ending = 0;
while ending == 0
  line = read(results_file,1,1,"(A)");
  if line == " CX"
    CX = read(results_file,1,length(AL));
  elseif line == " CZ"
    CZ = read(results_file,1,length(AL));
  elseif line == " CM"
    CM = read(results_file,1,length(AL));
  elseif line == " ORIGIN"
    origin = read(results_file,1,3);
  elseif line == " S"
    komp = komp+1;  // be careful!! this is the first component dependent field and counts them
    if komp == 3
      ending = 1;
      break
    end
    S(:,:,komp) = read(results_file,N(komp),M(komp));
  elseif line == " CP"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      cp(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " V"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      v(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " DIPOLE"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      doublet(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " P_STAT"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      p_stat(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " P_DYNA"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      p_dyna(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " P_MANO"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      p_mano(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " VX"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      vx(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " VY"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      vy(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " VZ"
    for i = 1:length(AL)
      read(results_file,1,1,"(A)");
      vz(:,:,komp,i) = read(results_file,N(komp),M(komp));
    end
  elseif line == " end"
    ending = 1;
  end
end

file('close',results_file)

endfunction
