// Copyright (C) 2009  Daniel Filkovic

// This file is part of APAME-WCS version 3.0.

// APAME-WCS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME-WCS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME-WCS.  If not, see <http://www.gnu.org/licenses/>.

// APAME-WCS - Aircraft PAnel MEthod-Wing Coefficients Solver
// - Pre and Post-Processor coupled with APAME solver for Simple wing
//   form analysis
//
// Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
//             - Kesic; Osnove aerodinamike
//             - Jankovic; Mehanika leta zrakoplova
//             - Abbott, Doenhoff; Theory of wing sections
//
// file root_tip.sci

function [xp1,zp1,xp2,zp2] = root_tip(xf1,zf1,xf2,zf2,croot,ctip,tipAL,span,lambda14,err);

// scaling and translating root airfoil
xp1 = (xf1-0.25)*croot;
zp1 = zf1*croot;

// scaling and translating tip airfoil
xp2temp = (xf2-0.25)*ctip;
zp2temp = zf2*ctip;

// tip airfoil rotation due to its angle of attack
xp2 = zeros(size(xp2temp));
zp2 = zeros(size(zp2temp));
if abs(tipAL) < err
  xp2 = xp2temp
  zp2 = zp2temp
else
  for i = 1:N1
    xp2(i) =  xp2temp(i)*cos(tipAL)+zp2temp(i)*sin(tipAL);
    zp2(i) = -xp2temp(i)*sin(tipAL)+zp2temp(i)*cos(tipAL);
  end
end

// translation of tip airfoil due to wing sweep
xp2 = xp2+(span/2)*tan(lambda14);

endfunction
