// Copyright (C) 2009  Daniel Filkovic

// This file is part of APAME-WCS version 3.0.
// This file is based on code from Katz, Plotkin: Low Speed Aerodynamics

// APAME-WCS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME-WCS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME-WCS.  If not, see <http://www.gnu.org/licenses/>.

// APAME-WCS - Aircraft PAnel MEthod-Wing Coefficients Solver
// - Pre and Post-Processor coupled with APAME solver for Simple wing
//   form analysis
//
// Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
//             - Kesic; Osnove aerodinamike
//             - Jankovic; Mehanika leta zrakoplova
//             - Abbott, Doenhoff; Theory of wing sections
//
// file vooren.sci

function [x,y] = vooren(N,N1,N2,thick,tr_ang);

pi = 3.141592653589793;

tr_ang = tr_ang*pi/180;             // airfoil trailing angle
div = 2*pi/N;                       // circle division

tr_ang_coef = 2-tr_ang/pi;          // airfoil trailing angle coefficient
rad = (1+thick)^(tr_ang_coef-1) / 2^tr_ang_coef;    // f-plane circle radius

x = zeros(N1,1);
y = zeros(N1,1);
x(1) = 1;
x(N1) = 1;
for i = 2:N
    TH = (i-1)*div;
    R1=sqrt((rad*(cos(TH)-1))^2+(rad*sin(TH))^2);
    R2=sqrt((rad*(cos(TH)-thick))^2+(rad*sin(TH))^2);
    
    TH1 = atan(rad*sin(TH),rad*(cos(TH)-1));
    TH2 = atan(rad*sin(TH),rad*(cos(TH)-thick));
    
    com=((R1^tr_ang_coef)/(R2^(tr_ang_coef-1)));
    x(N2-i)=com*(cos(tr_ang_coef*TH1)*cos((tr_ang_coef-1)*TH2)+sin(tr_ang_coef*TH1)*sin((tr_ang_coef-1)*TH2))+1;
    y(N2-i)=com*(sin(tr_ang_coef*TH1)*cos((tr_ang_coef-1)*TH2)-cos(tr_ang_coef*TH1)*sin((tr_ang_coef-1)*TH2));
end

endfunction
