// Copyright (C) 2009  Daniel Filkovic

// This file is part of APAME-WCS version 3.0.

// APAME-WCS is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// APAME-WCS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with APAME-WCS.  If not, see <http://www.gnu.org/licenses/>.

// APAME-WCS - Aircraft PAnel MEthod-Wing Coefficients Solver
// - Pre and Post-Processor coupled with APAME solver for Simple wing
//   form analysis
//
// Reference:  - Katz, Plotkin; Low Speed Aerodynamics - Second edition
//             - Kesic; Osnove aerodinamike
//             - Jankovic; Mehanika leta zrakoplova
//             - Abbott, Doenhoff; Theory of wing sections
//
// file write_input.sci

function write_input(find_ac,origin,V,ro,p_ref,mach,AL,span,c,Sw,method,farpoint,err,farfieldfaktor,collcalc,velorder,velometh,N1,M1,x,y,z);

pi = 3.1415926535897932;

Np1 = N/2+1;

input_file = file('open','wing.inp','unknown');

write(input_file,'APAME input file')
write(input_file,'VERSION 2.2')
write(input_file,('AIRSPEED '+string(V)))
write(input_file,('DENSITY '+string(ro)))
write(input_file,('PRESSURE '+string(p_ref)))
write(input_file,('MACH '+string(mach)))
write(input_file,('ALFA '+string(length(AL))))
for i = 1:length(AL)
    write(input_file,AL(i)*180/pi,'(1x,E14.8,$)')
end
write(input_file,'')
write(input_file,('BETA 1'))
write(input_file,'0')
write(input_file,('WINGSPAN '+string(span)))
write(input_file,('MAC '+string(c)))
write(input_file,('SURFACE '+string(Sw)))
write(input_file,('FIND_AC '+string(find_ac)))
write(input_file,[origin(1), origin(2),origin(3)])
write(input_file,('METHOD '+string(method)))
write(input_file,('WAKE '+string(farpoint)))
write(input_file,('ERROR '+string(err)))
write(input_file,('FARFIELD '+string(farfieldfaktor)))
write(input_file,('COLLCALC '+string(collcalc)))
write(input_file,('VELORDER '+string(velorder)))
write(input_file,('VELOMETH '+string(velometh)))
write(input_file,' RESULTS 1','(a10)')
write(input_file,' 1 0 0 0 1 1 0 0 1 1 1 1 1 1 1','(a30)')
write(input_file,('KOMP 4'))
write(input_file,[N1,M1],'(1x,i10,1x,i10)')

write(input_file,('left_wing '+string(N1)+' '+string(M1)+' '+string(1)))
for i=1:N1
  for j=1:M1
    write(input_file,x(i,j,1),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:N1
  for j=1:M1
    write(input_file,y(i,j,1),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:N1
  for j=1:M1
    write(input_file,z(i,j,1),'(1x,E14.7,$)')
  end
  write(input_file,'')
end

write(input_file,('right_wing '+string(N1)+' '+string(M1)+' '+string(1)))
for i=1:N1
  for j=1:M1
    write(input_file,x(i,j,2),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:N1
  for j=1:M1
    write(input_file,y(i,j,2),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:N1
  for j=1:M1
    write(input_file,z(i,j,2),'(1x,E14.7,$)')
  end
  write(input_file,'')
end

write(input_file,('left_wingtip '+string(Np1)+' '+string(2)+' '+string(0)))
for i=1:Np1
  for j=1:2
    write(input_file,x(i,j,3),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:Np1
  for j=1:2
    write(input_file,y(i,j,3),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:Np1
  for j=1:2
    write(input_file,z(i,j,3),'(1x,E14.7,$)')
  end
  write(input_file,'')
end

write(input_file,('right_wingtip '+string(Np1)+' '+string(2)+' '+string(0)))
for i=1:Np1
  for j=1:2
    write(input_file,x(i,j,4),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:Np1
  for j=1:2
    write(input_file,y(i,j,4),'(1x,E14.7,$)')
  end
  write(input_file,'')
end
for i=1:Np1
  for j=1:2
    write(input_file,z(i,j,4),'(1x,E14.7,$)')
  end
  write(input_file,'')
end

write(input_file,'# end of input file','(a19)')

file('close',input_file)

endfunction
