! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file apame.f90

program apame

implicit none

character*100 file_name,line_alfa,line_beta
character*50, dimension(:), allocatable ::  component_name
integer i,j,k,l,n,numalfa,numbeta,component_number,key_err,&
        &max_row,max_row1,max_row2,max_column,max_column1,max_panel_number,panel_total,&
        &info_solver,interactive,case_number,res_req,collcalc,method,velorder,velometh,&
        &find_ac,lwork,ac_count,info
integer, dimension(15) :: res_ind
real    speed,ro,mach,p_ref,pi,wing_span,mac,wing_surf,wake,error,farfield,start_time,origin3
real, dimension(2) :: cmalfa,cmalfa2,originx,originz,temp2
real, dimension(3) :: origin,origin_old
real, dimension(6) :: work
integer,    dimension(:),       allocatable :: row_number,column_number,panel_number,row_number1,column_number1,lifting
integer,    dimension(:,:),     allocatable :: ipiv
real,       dimension(:),       allocatable :: alfa,beta,speed_x,speed_y,speed_z,&
        &coef_x,coef_y,coef_z,coef_l,coef_m,coef_n,Fx,Fy,Fz,Fl,Fm,Fn,coef_m_ac
real,       dimension(:,:),     allocatable :: a,b,sigma,rhs,&
        &coef_x_comp,coef_y_comp,coef_z_comp,coef_l_comp,coef_m_comp,coef_n_comp,&
        &Fx_comp,Fy_comp,Fz_comp,Fl_comp,Fm_comp,Fn_comp,ac_matrix1,ac_matrix2
real,       dimension(:,:,:),   allocatable :: FF,S,gama_vector,sigm_vector,&
        &   x,y,z,  u1,u2,u3,   o1,o2,o3,   n1,n2,n3,   p1,p2,p3,   cx,cy,cz,   x1,x2,x3,x4,    y1,y2,y3,y4,    d1,d2,d3,d4
real,       dimension(:,:,:,:), allocatable :: gama,sigm,cp,v,vx,vy,vz,p_dyna,p_mano,p_stat

parameter (pi=0.31415927E+01)
parameter (lwork=6)

! READING APAME INPUT FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! getting command line argument
call getarg(1,file_name)
interactive = 0
if (file_name .eq. '') then     ! in case no input file is stated at command line, APAME will not display any messages on the screen
    interactive = 1
    call title
    read *, file_name
endif

call tic(start_time)

call new_line(interactive)

! open input file for usage
open(unit=1,file = trim(file_name)//'.inp',status='old')
! create (replace) log file
open(unit=2,file = trim(file_name)//'.log',status='replace')

! print date and time
call write_time(interactive)
call new_line(interactive)

call message(interactive,"Reading APAME input file...")

call read_key(interactive,speed,ro,p_ref,mach,numalfa,line_alfa,numbeta,line_beta,wing_span,mac,wing_surf,origin,method,&
    &wake,error,farfield,collcalc,velorder,velometh,find_ac,res_req,res_ind,component_number,&
    &max_row,max_row1,max_row2,max_column,max_column1,key_err)
if (key_err .eq. 1) then
    goto 999
endif

allocate (alfa(numalfa),beta(numbeta))
call read_ang(line_alfa,line_beta,numalfa,numbeta,alfa,beta)

allocate (component_name(component_number),lifting(component_number),column_number1(component_number),row_number1(component_number))
allocate (x(max_row2,max_column1,component_number),&
         &y(max_row2,max_column1,component_number),&
         &z(max_row2,max_column1,component_number))
call read_geom(max_row2,max_column1,component_number,x,y,z,component_name,row_number1,column_number1,lifting)
close (1)

alfa = alfa*pi/180
beta = beta*pi/180

! calculating case numbers
call cases(numalfa,numbeta,case_number)

! calculating airspeed components
allocate (speed_x(case_number),speed_y(case_number),speed_z(case_number))
call speeds(case_number,numalfa,numbeta,speed,alfa,beta,speed_x,speed_y,speed_z)

! calculating number of panels by components and total number of panels
allocate (row_number(component_number),column_number(component_number),panel_number (component_number))
call pan_numb(component_number,row_number1,column_number1,row_number,column_number,panel_number,max_panel_number,panel_total)

call message(interactive,"Done!")
call new_line(interactive)
call new_line(interactive)

call write_ptot(interactive,panel_total)

call write_key(interactive,speed,ro,p_ref,mach,wing_span,mac,wing_surf,origin,method,&
    &wake,error,farfield,collcalc,velorder,velometh,find_ac,component_number)

call write_case(interactive,numalfa,numbeta,alfa,beta)
call new_line(interactive)

call message(interactive,"Calculating grid information...")

! CALCULATION OF GRID INFORMATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allocate (FF(max_row1,max_column,component_number),S(max_row1,max_column,component_number),&
    &u1(max_row1,max_column,component_number),u2(max_row1,max_column,component_number),u3(max_row1,max_column,component_number),&
    &o1(max_row1,max_column,component_number),o2(max_row1,max_column,component_number),o3(max_row1,max_column,component_number),&
    &n1(max_row1,max_column,component_number),n2(max_row1,max_column,component_number),n3(max_row1,max_column,component_number),&
    &p1(max_row1,max_column,component_number),p2(max_row1,max_column,component_number),p3(max_row1,max_column,component_number),&
    &cx(max_row1,max_column,component_number),cy(max_row1,max_column,component_number),cz(max_row1,max_column,component_number),&
    &x1(max_row1,max_column,component_number),y1(max_row1,max_column,component_number),&
    &x2(max_row1,max_column,component_number),y2(max_row1,max_column,component_number),&
    &x3(max_row1,max_column,component_number),y3(max_row1,max_column,component_number),&
    &x4(max_row1,max_column,component_number),y4(max_row1,max_column,component_number),&
    &d1(max_row1,max_column,component_number),d2(max_row1,max_column,component_number),&
    &d3(max_row1,max_column,component_number),d4(max_row1,max_column,component_number))
do i=1,component_number
    call grid(FF(1:max_row1,1:max_column,i),S(1:max_row1,1:max_column,i),&
    &u1(1:max_row1,1:max_column,i),u2(1:max_row1,1:max_column,i),u3(1:max_row1,1:max_column,i),&
    &o1(1:max_row1,1:max_column,i),o2(1:max_row1,1:max_column,i),o3(1:max_row1,1:max_column,i),&
    &n1(1:max_row1,1:max_column,i),n2(1:max_row1,1:max_column,i),n3(1:max_row1,1:max_column,i),&
    &p1(1:max_row1,1:max_column,i),p2(1:max_row1,1:max_column,i),p3(1:max_row1,1:max_column,i),&
    &cx(1:max_row1,1:max_column,i),cy(1:max_row1,1:max_column,i),cz(1:max_row1,1:max_column,i),&
    &x1(1:max_row1,1:max_column,i),y1(1:max_row1,1:max_column,i),x2(1:max_row1,1:max_column,i),y2(1:max_row1,1:max_column,i),&
    &x3(1:max_row1,1:max_column,i),y3(1:max_row1,1:max_column,i),x4(1:max_row1,1:max_column,i),y4(1:max_row1,1:max_column,i),&
    &d1(1:max_row1,1:max_column,i),d2(1:max_row1,1:max_column,i),d3(1:max_row1,1:max_column,i),d4(1:max_row1,1:max_column,i),&
    &x(1:max_row2,1:max_column1,i),y(1:max_row2,1:max_column1,i),z(1:max_row2,1:max_column1,i),&
    &max_column,max_column1,max_row1,max_row2,wake,error,farfield,collcalc)
enddo
call message(interactive,"Done!")
call new_line(interactive)
call new_line(interactive)

call toc(interactive,start_time,"    Time for pre-processing:")
call new_line(interactive)

call tic(start_time)
! CALCULATION OF INFLUENCE COEFFICIENTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
call message(interactive,"Calculating influence coefficients...")
if (method .eq. 0) then
    allocate (a(panel_total,panel_total),b(panel_total,panel_total))
    k = 1
    do i=1,component_number
        l = 1
        do j=1,component_number
            call comp_infl_const_doubl_sourc(i,j,&
                &a(k:k+panel_number(i)-1,l:l+panel_number(j)-1),&
                &b(k:k+panel_number(i)-1,l:l+panel_number(j)-1),&
                &FF(1:row_number1(j),1:column_number(j),j),error,&
                &S(1:row_number1(j),1:column_number(j),j),&
                &column_number(j),row_number(j),row_number1(j),panel_number(j),&
                &u1(1:row_number1(j),1:column_number(j),j),&
                &u2(1:row_number1(j),1:column_number(j),j),&
                &u3(1:row_number1(j),1:column_number(j),j),&
                &o1(1:row_number1(j),1:column_number(j),j),&
                &o2(1:row_number1(j),1:column_number(j),j),&
                &o3(1:row_number1(j),1:column_number(j),j),&
                &n1(1:row_number1(j),1:column_number(j),j),&
                &n2(1:row_number1(j),1:column_number(j),j),&
                &n3(1:row_number1(j),1:column_number(j),j),&
                &cx(1:row_number1(j),1:column_number(j),j),&
                &cy(1:row_number1(j),1:column_number(j),j),&
                &cz(1:row_number1(j),1:column_number(j),j),&
                &x1(1:row_number1(j),1:column_number(j),j),&
                &y1(1:row_number1(j),1:column_number(j),j),&
                &x2(1:row_number1(j),1:column_number(j),j),&
                &y2(1:row_number1(j),1:column_number(j),j),&
                &x3(1:row_number1(j),1:column_number(j),j),&
                &y3(1:row_number1(j),1:column_number(j),j),&
                &x4(1:row_number1(j),1:column_number(j),j),&
                &y4(1:row_number1(j),1:column_number(j),j),&
                &d1(1:row_number1(j),1:column_number(j),j),&
                &d2(1:row_number1(j),1:column_number(j),j),&
                &d3(1:row_number1(j),1:column_number(j),j),&
                &d4(1:row_number1(j),1:column_number(j),j),&
                &column_number(i),row_number(i),row_number1(i),panel_number(i),&
                &cx(1:row_number1(i),1:column_number(i),i),&
                &cy(1:row_number1(i),1:column_number(i),i),&
                &cz(1:row_number1(i),1:column_number(i),i))
            l = l+panel_number(j)
        enddo
        k = k+panel_number(i)
    enddo
elseif (method .eq. 1) then
    allocate (a(panel_total,panel_total))
    k = 1
    do i=1,component_number
        l = 1
        do j=1,component_number
            call comp_infl_const_doubl(i,j,&
                &a(k:k+panel_number(i)-1,l:l+panel_number(j)-1),&
                &FF(1:row_number1(j),1:column_number(j),j),error,&
                &S(1:row_number1(j),1:column_number(j),j),&
                &column_number(j),row_number(j),row_number1(j),panel_number(j),&
                &u1(1:row_number1(j),1:column_number(j),j),&
                &u2(1:row_number1(j),1:column_number(j),j),&
                &u3(1:row_number1(j),1:column_number(j),j),&
                &o1(1:row_number1(j),1:column_number(j),j),&
                &o2(1:row_number1(j),1:column_number(j),j),&
                &o3(1:row_number1(j),1:column_number(j),j),&
                &n1(1:row_number1(j),1:column_number(j),j),&
                &n2(1:row_number1(j),1:column_number(j),j),&
                &n3(1:row_number1(j),1:column_number(j),j),&
                &cx(1:row_number1(j),1:column_number(j),j),&
                &cy(1:row_number1(j),1:column_number(j),j),&
                &cz(1:row_number1(j),1:column_number(j),j),&
                &x1(1:row_number1(j),1:column_number(j),j),&
                &y1(1:row_number1(j),1:column_number(j),j),&
                &x2(1:row_number1(j),1:column_number(j),j),&
                &y2(1:row_number1(j),1:column_number(j),j),&
                &x3(1:row_number1(j),1:column_number(j),j),&
                &y3(1:row_number1(j),1:column_number(j),j),&
                &x4(1:row_number1(j),1:column_number(j),j),&
                &y4(1:row_number1(j),1:column_number(j),j),&
                &d1(1:row_number1(j),1:column_number(j),j),&
                &d2(1:row_number1(j),1:column_number(j),j),&
                &d3(1:row_number1(j),1:column_number(j),j),&
                &d4(1:row_number1(j),1:column_number(j),j),&
                &column_number(i),row_number(i),row_number1(i),panel_number(i),&
                &cx(1:row_number1(i),1:column_number(i),i),&
                &cy(1:row_number1(i),1:column_number(i),i),&
                &cz(1:row_number1(i),1:column_number(i),i))
            l = l+panel_number(j)
        enddo
        k = k+panel_number(i)
    enddo
endif

! calculating wake influence coefficients
k = 1
do i=1,component_number
    l = 1
    do j=1,component_number
        if (lifting(j) .eq. 1) then
            call comp_infl_const_doubl_wake(&
                &a(k:k+panel_number(i)-1,l:l+panel_number(j)-1),&
                &FF(1:row_number1(j),1:column_number(j),j),error,&
                &S(1:row_number1(j),1:column_number(j),j),&
                &column_number(j),row_number(j),row_number1(j),panel_number(j),&
                &u1(1:row_number1(j),1:column_number(j),j),&
                &u2(1:row_number1(j),1:column_number(j),j),&
                &u3(1:row_number1(j),1:column_number(j),j),&
                &o1(1:row_number1(j),1:column_number(j),j),&
                &o2(1:row_number1(j),1:column_number(j),j),&
                &o3(1:row_number1(j),1:column_number(j),j),&
                &n1(1:row_number1(j),1:column_number(j),j),&
                &n2(1:row_number1(j),1:column_number(j),j),&
                &n3(1:row_number1(j),1:column_number(j),j),&
                &cx(1:row_number1(j),1:column_number(j),j),&
                &cy(1:row_number1(j),1:column_number(j),j),&
                &cz(1:row_number1(j),1:column_number(j),j),&
                &x1(1:row_number1(j),1:column_number(j),j),&
                &y1(1:row_number1(j),1:column_number(j),j),&
                &x2(1:row_number1(j),1:column_number(j),j),&
                &y2(1:row_number1(j),1:column_number(j),j),&
                &x3(1:row_number1(j),1:column_number(j),j),&
                &y3(1:row_number1(j),1:column_number(j),j),&
                &x4(1:row_number1(j),1:column_number(j),j),&
                &y4(1:row_number1(j),1:column_number(j),j),&
                &d1(1:row_number1(j),1:column_number(j),j),&
                &d2(1:row_number1(j),1:column_number(j),j),&
                &d3(1:row_number1(j),1:column_number(j),j),&
                &d4(1:row_number1(j),1:column_number(j),j),&
                &column_number(i),row_number(i),row_number1(i),panel_number(i),&
                &cx(1:row_number1(i),1:column_number(i),i),&
                &cy(1:row_number1(i),1:column_number(i),i),&
                &cz(1:row_number1(i),1:column_number(i),i))
        endif
        l = l+panel_number(j)
    enddo
    k = k+panel_number(i)
enddo
deallocate (d1,d2,d3,d4,x1,x2,x3,x4,y1,y2,y3,y4)
call message(interactive,"Done!")
call new_line(interactive)
call new_line(interactive)

call toc(interactive,start_time,"    Time for calculating influence coefficients:")
call new_line(interactive)

call tic(start_time)
! SOLVING SYSTEM OF EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
call message(interactive,"Solving system of equations...")

if (method .eq. 0) then
    allocate (sigma(panel_total,case_number))
    ! calculating source strengths
    do k=1,case_number
        j = 1
        do i=1,component_number
            call calculate_sigma(sigma(j:j+panel_number(i)-1,k),speed_x(k),speed_y(k),speed_z(k),&
            &column_number(i),row_number(i),panel_number(i),&
            &n1(1:row_number(i),1:column_number(i),i),&
            &n2(1:row_number(i),1:column_number(i),i),&
            &n3(1:row_number(i),1:column_number(i),i))
            j = j+panel_number(i)
        enddo
    enddo
    ! calculating "right hand side"
    allocate (rhs(panel_total,case_number))
    do k=1,case_number
        do i=1,panel_total
            rhs(i,k) = 0
            do j=1,panel_total
                rhs(i,k) = rhs(i,k)+b(i,j)*sigma(j,k)
            enddo
        enddo
    enddo
    deallocate (b)
elseif (method .eq. 1) then
    allocate (rhs(panel_total,case_number))
    ! calculating "right hand side"
    do k=1,case_number
        j = 1
        do i=1,component_number
            call rhs_const_sourc(rhs(j:j+panel_number(i)-1,k),speed_x(k),speed_y(k),speed_z(k),&
            &column_number(i),row_number(i),panel_number(i),&
            &cx(1:row_number(i),1:column_number(i),i),&
            &cy(1:row_number(i),1:column_number(i),i),&
            &cz(1:row_number(i),1:column_number(i),i))
            j = j+panel_number(i)
        enddo
    enddo
endif

! solving system of equations
allocate (ipiv(panel_total,panel_total))
call sgesv(panel_total,case_number,a,panel_total,ipiv,rhs,panel_total,info_solver)
deallocate (a,ipiv)
call message(interactive,"Done!")
call new_line(interactive)

if (info_solver .lt. 0) then
    call message(interactive,"    ERROR: Illegal value detected in system solver")
    goto 999
elseif (info_solver .gt. 0) then
    call message(interactive,"    ERROR: Factorization complete with singular solution")
    goto 999
endif

call new_line(interactive)
call toc(interactive,start_time,"    Time for solving system of equations:")
call new_line(interactive)

call tic(start_time)
! distribution of dipole values to components
call message(interactive,"Calculating results data...")

allocate (gama_vector(max_panel_number,component_number,case_number))
do k=1,case_number
    j = 1
    do i=1,component_number
        gama_vector(1:panel_number(i),i,k) = rhs(j:j+panel_number(i)-1,k)
        j = j+panel_number(i)
    enddo
enddo
deallocate (rhs)

! returning dipole values to i,j matrix
allocate (gama(max_row,max_column,component_number,case_number))
do n=1,case_number
    do i=1,component_number
    j = 0
        do k=1,row_number(i)
            do l=1,column_number(i)
                j = j+1
                gama(k,l,i,n) = gama_vector(j,i,n)
            enddo
        enddo
    enddo
enddo
deallocate (gama_vector)

if (method .eq. 0) then
    ! distribution of source values to components
    allocate (sigm_vector(max_panel_number,component_number,case_number))
    do k=1,case_number
        j = 1
        do i=1,component_number
            sigm_vector(1:panel_number(i),i,k) = sigma(j:j+panel_number(i)-1,k)
            j = j+panel_number(i)
        enddo
    enddo
    deallocate (sigma)
    
    ! returning source values to i,j matrix
    allocate (sigm(max_row,max_column,component_number,case_number))
    do n=1,case_number
        do i=1,component_number
            j = 0
            do k=1,row_number(i)
                do l=1,column_number(i)
                    j = j+1
                    sigm(k,l,i,n) = sigm_vector(j,i,n)
                enddo
            enddo
        enddo
    enddo
    deallocate (sigm_vector)
endif

! CALCULATION OF INDUCED VELOCITIES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allocate (&
    & v(max_row,max_column,component_number,case_number),&
    &vx(max_row,max_column,component_number,case_number),&
    &vy(max_row,max_column,component_number,case_number),&
    &vz(max_row,max_column,component_number,case_number))
v = 0.
vx = 0.
vy = 0.
vz = 0.
do j=1,case_number
    do i=1,component_number
        if (row_number(i) .gt. 1 .and. column_number(i) .gt. 1) then
            if (velometh .eq. 0) then
                call calculate_velocities_directional(&
                & v(1:row_number(i),1:column_number(i),i,j),&
                &vx(1:row_number(i),1:column_number(i),i,j),&
                &vy(1:row_number(i),1:column_number(i),i,j),&
                &vz(1:row_number(i),1:column_number(i),i,j),&
                &method,velorder,&
                &gama(1:row_number(i),1:column_number(i),i,j),&
                &speed_x(j),speed_y(j),speed_z(j),&
                &column_number(i),row_number(i),&
                &u1(1:row_number(i),1:column_number(i),i),&
                &u2(1:row_number(i),1:column_number(i),i),&
                &u3(1:row_number(i),1:column_number(i),i),&
                &p1(1:row_number(i),1:column_number(i),i),&
                &p2(1:row_number(i),1:column_number(i),i),&
                &p3(1:row_number(i),1:column_number(i),i),&
                &o1(1:row_number(i),1:column_number(i),i),&
                &o2(1:row_number(i),1:column_number(i),i),&
                &o3(1:row_number(i),1:column_number(i),i),&
                &cx(1:row_number(i),1:column_number(i),i),&
                &cy(1:row_number(i),1:column_number(i),i),&
                &cz(1:row_number(i),1:column_number(i),i))
            elseif (velometh .eq. 1) then
                call calculate_velocities_surface(&
                & v(1:row_number(i),1:column_number(i),i,j),&
                &vx(1:row_number(i),1:column_number(i),i,j),&
                &vy(1:row_number(i),1:column_number(i),i,j),&
                &vz(1:row_number(i),1:column_number(i),i,j),&
                &method,velorder,&
                &gama(1:row_number(i),1:column_number(i),i,j),&
                &speed_x(j),speed_y(j),speed_z(j),&
                &column_number(i),row_number(i),&
                &u1(1:row_number(i),1:column_number(i),i),&
                &u2(1:row_number(i),1:column_number(i),i),&
                &u3(1:row_number(i),1:column_number(i),i),&
                &o1(1:row_number(i),1:column_number(i),i),&
                &o2(1:row_number(i),1:column_number(i),i),&
                &o3(1:row_number(i),1:column_number(i),i),&
                &cx(1:row_number(i),1:column_number(i),i),&
                &cy(1:row_number(i),1:column_number(i),i),&
                &cz(1:row_number(i),1:column_number(i),i))
            endif
        endif
    enddo
enddo

! CALCULATION OF PRESSURES AND FORCES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allocate (&
    &coef_m_ac(numalfa),ac_matrix1(numalfa,2),ac_matrix2(numalfa,3),&
    &coef_x(case_number),coef_y(case_number),coef_z(case_number),&
    &coef_l(case_number),coef_m(case_number),coef_n(case_number),&
    &Fx(case_number),Fy(case_number),Fz(case_number),&
    &Fl(case_number),Fm(case_number),Fn(case_number),&
    &cp(max_row,max_column,component_number,case_number),&
    &p_dyna(max_row,max_column,component_number,case_number),&
    &p_mano(max_row,max_column,component_number,case_number),&
    &p_stat(max_row,max_column,component_number,case_number),&
    &coef_x_comp(component_number,case_number),&
    &coef_y_comp(component_number,case_number),&
    &coef_z_comp(component_number,case_number),&
    &coef_l_comp(component_number,case_number),&
    &coef_m_comp(component_number,case_number),&
    &coef_n_comp(component_number,case_number),&
    &Fx_comp(component_number,case_number),&
    &Fy_comp(component_number,case_number),&
    &Fz_comp(component_number,case_number),&
    &Fl_comp(component_number,case_number),&
    &Fm_comp(component_number,case_number),&
    &Fn_comp(component_number,case_number))
if (find_ac .eq. 1 .and. numalfa .gt. 1) then
    ! saving old origin in case new one does not converge
    origin_old = origin
    ! routine for finding and using AC
    do ac_count=1,10
        temp2 = (/ -mac/2,mac/2 /)
        do k=1,2
            do j=1,numalfa
                ac_matrix1(j,1) = alfa(j)
                ac_matrix1(j,2) = 1
                coef_m_ac(j) = 0.
                do i=1,component_number
                    call calculate_pressures_force(&
                    &cp(1:row_number(i),1:column_number(i),i,j),&
                    &p_dyna(1:row_number(i),1:column_number(i),i,j),&
                    &p_mano(1:row_number(i),1:column_number(i),i,j),&
                    &p_stat(1:row_number(i),1:column_number(i),i,j),&
                    &Fx_comp(i,j),Fy_comp(i,j),Fz_comp(i,j),Fl_comp(i,j),Fm_comp(i,j),Fn_comp(i,j),&
                    &coef_x_comp(i,j),coef_y_comp(i,j),coef_z_comp(i,j),coef_l_comp(i,j),coef_m_comp(i,j),coef_n_comp(i,j),&
                    &mac,wing_span,wing_surf,origin+(/temp2(k),0.,0./),&
                    &speed,mach,ro,p_ref,&
                    &S(1:row_number(i),1:column_number(i),i),&
                    &column_number(i),row_number(i),&
                    & v(1:row_number(i),1:column_number(i),i,j),&
                    &n1(1:row_number(i),1:column_number(i),i),&
                    &n2(1:row_number(i),1:column_number(i),i),&
                    &n3(1:row_number(i),1:column_number(i),i),&
                    &cx(1:row_number(i),1:column_number(i),i),&
                    &cy(1:row_number(i),1:column_number(i),i),&
                    &cz(1:row_number(i),1:column_number(i),i))
                    
                    coef_m_ac(j) = coef_m_ac(j) + coef_m_comp(i,j)
                enddo
            enddo
            call sgels('N', numalfa, 2, 1, ac_matrix1, numalfa, coef_m_ac, numalfa, work, lwork, info)
            cmalfa(k) = coef_m_ac(1)
            originx(k) = origin(1)+temp2(k)
        enddo
        origin(1) = (cmalfa(1)*originx(2)-cmalfa(2)*originx(1))/(cmalfa(1)-cmalfa(2))
        
        do k=1,2
            do j=1,numalfa
                ac_matrix2(j,1) = alfa(j)**2
                ac_matrix2(j,2) = alfa(j)
                ac_matrix2(j,3) = 1
                coef_m_ac(j) = 0.
                do i=1,component_number
                    call calculate_pressures_force(&
                    &cp(1:row_number(i),1:column_number(i),i,j),&
                    &p_dyna(1:row_number(i),1:column_number(i),i,j),&
                    &p_mano(1:row_number(i),1:column_number(i),i,j),&
                    &p_stat(1:row_number(i),1:column_number(i),i,j),&
                    &Fx_comp(i,j),Fy_comp(i,j),Fz_comp(i,j),Fl_comp(i,j),Fm_comp(i,j),Fn_comp(i,j),&
                    &coef_x_comp(i,j),coef_y_comp(i,j),coef_z_comp(i,j),coef_l_comp(i,j),coef_m_comp(i,j),coef_n_comp(i,j),&
                    &mac,wing_span,wing_surf,origin+(/0.,0.,temp2(k)/),&
                    &speed,mach,ro,p_ref,&
                    &S(1:row_number(i),1:column_number(i),i),&
                    &column_number(i),row_number(i),&
                    & v(1:row_number(i),1:column_number(i),i,j),&
                    &n1(1:row_number(i),1:column_number(i),i),&
                    &n2(1:row_number(i),1:column_number(i),i),&
                    &n3(1:row_number(i),1:column_number(i),i),&
                    &cx(1:row_number(i),1:column_number(i),i),&
                    &cy(1:row_number(i),1:column_number(i),i),&
                    &cz(1:row_number(i),1:column_number(i),i))

                    coef_m_ac(j) = coef_m_ac(j) + coef_m_comp(i,j)
                enddo
            enddo
            call sgels('N', numalfa, 3, 1, ac_matrix2, numalfa, coef_m_ac, numalfa, work, lwork, info)
            cmalfa2(k) = coef_m_ac(1)
            originz(k) = origin(3)+temp2(k)
        enddo
        origin3 = (cmalfa2(1)*originz(2)-cmalfa2(2)*originz(1))/(cmalfa2(1)-cmalfa2(2))
        if (abs(origin(3)-origin3)/abs(origin3) .lt. 100*error) then
            origin(3) = origin3
            goto 800
        endif
        origin(3) = origin3
    enddo
    ! if no convergence
    call message(interactive," WARNING: Aerodynamic center could not be found within 10 iterations, using >>origin<<...")
    origin = origin_old
800 continue
    ! calculating all forces and moments using new origin (old for no convergence)
    do j=1,case_number
        coef_x(j) = 0.
        coef_y(j) = 0.
        coef_z(j) = 0.
        coef_l(j) = 0.
        coef_m(j) = 0.
        coef_n(j) = 0.
        Fx(j) = 0.
        Fy(j) = 0.
        Fz(j) = 0.
        Fl(j) = 0.
        Fm(j) = 0.
        Fn(j) = 0.
        do i=1,component_number
            call calculate_pressures_force(&
            &cp(1:row_number(i),1:column_number(i),i,j),&
            &p_dyna(1:row_number(i),1:column_number(i),i,j),&
            &p_mano(1:row_number(i),1:column_number(i),i,j),&
            &p_stat(1:row_number(i),1:column_number(i),i,j),&
            &Fx_comp(i,j),Fy_comp(i,j),Fz_comp(i,j),Fl_comp(i,j),Fm_comp(i,j),Fn_comp(i,j),&
            &coef_x_comp(i,j),coef_y_comp(i,j),coef_z_comp(i,j),coef_l_comp(i,j),coef_m_comp(i,j),coef_n_comp(i,j),&
            &mac,wing_span,wing_surf,origin,&
            &speed,mach,ro,p_ref,&
            &S(1:row_number(i),1:column_number(i),i),&
            &column_number(i),row_number(i),&
            & v(1:row_number(i),1:column_number(i),i,j),&
            &n1(1:row_number(i),1:column_number(i),i),&
            &n2(1:row_number(i),1:column_number(i),i),&
            &n3(1:row_number(i),1:column_number(i),i),&
            &cx(1:row_number(i),1:column_number(i),i),&
            &cy(1:row_number(i),1:column_number(i),i),&
            &cz(1:row_number(i),1:column_number(i),i))

            coef_x(j) = coef_x(j) + coef_x_comp(i,j)
            coef_y(j) = coef_y(j) + coef_y_comp(i,j)
            coef_z(j) = coef_z(j) + coef_z_comp(i,j)
            coef_l(j) = coef_l(j) + coef_l_comp(i,j)
            coef_m(j) = coef_m(j) + coef_m_comp(i,j)
            coef_n(j) = coef_n(j) + coef_n_comp(i,j)
            Fx(j) = Fx(j) + Fx_comp(i,j)
            Fy(j) = Fy(j) + Fy_comp(i,j)
            Fz(j) = Fz(j) + Fz_comp(i,j)
            Fl(j) = Fl(j) + Fl_comp(i,j)
            Fm(j) = Fm(j) + Fm_comp(i,j)
            Fn(j) = Fn(j) + Fn_comp(i,j)
        enddo
    enddo
else
    do j=1,case_number
        coef_x(j) = 0.
        coef_y(j) = 0.
        coef_z(j) = 0.
        coef_l(j) = 0.
        coef_m(j) = 0.
        coef_n(j) = 0.
        Fx(j) = 0.
        Fy(j) = 0.
        Fz(j) = 0.
        Fl(j) = 0.
        Fm(j) = 0.
        Fn(j) = 0.
        do i=1,component_number
            call calculate_pressures_force(&
            &cp(1:row_number(i),1:column_number(i),i,j),&
            &p_dyna(1:row_number(i),1:column_number(i),i,j),&
            &p_mano(1:row_number(i),1:column_number(i),i,j),&
            &p_stat(1:row_number(i),1:column_number(i),i,j),&
            &Fx_comp(i,j),Fy_comp(i,j),Fz_comp(i,j),Fl_comp(i,j),Fm_comp(i,j),Fn_comp(i,j),&
            &coef_x_comp(i,j),coef_y_comp(i,j),coef_z_comp(i,j),coef_l_comp(i,j),coef_m_comp(i,j),coef_n_comp(i,j),&
            &mac,wing_span,wing_surf,origin,&
            &speed,mach,ro,p_ref,&
            &S(1:row_number(i),1:column_number(i),i),&
            &column_number(i),row_number(i),&
            & v(1:row_number(i),1:column_number(i),i,j),&
            &n1(1:row_number(i),1:column_number(i),i),&
            &n2(1:row_number(i),1:column_number(i),i),&
            &n3(1:row_number(i),1:column_number(i),i),&
            &cx(1:row_number(i),1:column_number(i),i),&
            &cy(1:row_number(i),1:column_number(i),i),&
            &cz(1:row_number(i),1:column_number(i),i))

            coef_x(j) = coef_x(j) + coef_x_comp(i,j)
            coef_y(j) = coef_y(j) + coef_y_comp(i,j)
            coef_z(j) = coef_z(j) + coef_z_comp(i,j)
            coef_l(j) = coef_l(j) + coef_l_comp(i,j)
            coef_m(j) = coef_m(j) + coef_m_comp(i,j)
            coef_n(j) = coef_n(j) + coef_n_comp(i,j)
            Fx(j) = Fx(j) + Fx_comp(i,j)
            Fy(j) = Fy(j) + Fy_comp(i,j)
            Fz(j) = Fz(j) + Fz_comp(i,j)
            Fl(j) = Fl(j) + Fl_comp(i,j)
            Fm(j) = Fm(j) + Fm_comp(i,j)
            Fn(j) = Fn(j) + Fn_comp(i,j)
        enddo
    enddo
endif

call message(interactive,"Done!")
call new_line(interactive)
call new_line(interactive)
call toc(interactive,start_time,"    Time for results calculation:")
call new_line(interactive)

! PRINTING COEFFICIENTS
call write_coef(interactive,case_number,coef_x,coef_y,coef_z,coef_l,coef_m,coef_n)
call new_line(interactive)
call new_line(interactive)

! WRITING APAME RESULTS FILE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (res_req .eq. 1) then
    call results(interactive,file_name,speed,ro,p_ref,mach,&
        &numalfa,alfa,numbeta,beta,wing_span,mac,wing_surf,origin,method,wake,error,farfield,&
        &collcalc,velorder,velometh,find_ac,ac_count,&
        &component_number,component_name,lifting,case_number,x,y,z,cx,cy,cz,cp,v,vx,vy,vz,&
        &p_dyna,p_mano,p_stat,gama,sigm,S,FF,n1,n2,n3,u1,u2,u3,p1,p2,p3,o1,o2,o3,&
        &max_row,max_column,row_number,column_number,row_number1,column_number1,max_row1,max_row2,max_column1,&
        &coef_x,coef_y,coef_z,coef_l,coef_m,coef_n,&
        &Fx,Fy,Fz,Fl,Fm,Fn,&
        &Fx_comp,Fy_comp,Fz_comp,Fl_comp,Fm_comp,Fn_comp,&
        &coef_x_comp,coef_y_comp,coef_z_comp,coef_l_comp,coef_m_comp,coef_n_comp,res_ind)
endif

call message(interactive,"Job complete!")
call new_line(interactive)

999 call new_line(interactive)
close (2)
end
