! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! calculate_pressures_force.f90

subroutine calculate_pressures_force(cp,p_dyna,p_mano,p_stat,Fx,Fy,Fz,Fl,Fm,Fn,&
    &coef_x,coef_y,coef_z,coef_l,coef_m,coef_n,&
    &mac,wing_span,wing_surf,origin,speed,mach,ro,p_ref,S,column_number,row_number,v,&
    &n1,n2,n3,cx,cy,cz)

implicit none

integer, intent(in) :: column_number,row_number
real, intent(in) :: mac,wing_span,wing_surf,speed,mach,ro,p_ref
real, dimension(3), intent(in) :: origin
real, dimension(row_number,column_number), intent(in) :: v,S,n1,n2,n3,cx,cy,cz
real, intent(out) :: Fx,Fy,Fz,Fl,Fm,Fn,coef_x,coef_y,coef_z,coef_l,coef_m,coef_n
real, dimension(row_number,column_number), intent(out) :: cp,p_dyna,p_mano,p_stat
integer i,j
real q,dx,dy,dz,v_square,speed_square,beta

Fx = 0.
Fy = 0.
Fz = 0.
Fl = 0.
Fm = 0.
Fn = 0.
speed_square = speed**2
q = 0.5*ro*speed_square
if (mach .gt. 0.) then
    beta = sqrt(abs(1.-mach**2))
endif

do j=1,column_number
    do i=1,row_number
        v_square = v(i,j)**2
        
        ! calculating pressure coefficient
        if (mach .gt. 0.) then
            cp(i,j) = (1.-v_square/speed_square)/beta
        else
            cp(i,j) = 1.-v_square/speed_square
        endif
        
        ! calculating pressures
        p_dyna(i,j) = 0.5*ro*v_square
        p_mano(i,j) = cp(i,j)*q
        p_stat(i,j) = p_mano(i,j)+p_ref
        
        ! calculating force components
        dx = -p_mano(i,j)*S(i,j)*n1(i,j)
        dy = -p_mano(i,j)*S(i,j)*n2(i,j)
        dz = -p_mano(i,j)*S(i,j)*n3(i,j)
        
        ! summing force components
        Fx = Fx + dx
        Fy = Fy + dy
        Fz = Fz + dz
        
        Fl = Fl                          - dy*(cz(i,j)-origin(3)) + dz*(cy(i,j)-origin(2))
        Fm = Fm + dx*(cz(i,j)-origin(3))                          - dz*(cx(i,j)-origin(1))
        Fn = Fn - dx*(cy(i,j)-origin(2)) + dy*(cx(i,j)-origin(1))
    enddo
enddo

! calculating aerodynamic coefficients
coef_x = Fx/q/wing_surf
coef_y = Fy/q/wing_surf
coef_z = Fz/q/wing_surf
coef_l = Fl/q/wing_surf/wing_span
coef_m = Fm/q/wing_surf/mac
coef_n = Fn/q/wing_surf/wing_span

return
end subroutine

