! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! calculate_velocities_directional.f90

subroutine calculate_velocities_directional(v,vx,vy,vz,method,velorder,&
    &gama,speed_x,speed_y,speed_z,column_number,row_number,&
    &u1,u2,u3,p1,p2,p3,o1,o2,o3,cx,cy,cz)

implicit none

integer, intent(in) :: column_number,row_number,method,velorder
real, intent(in) :: speed_x,speed_y,speed_z
real, dimension(row_number,column_number), intent(in) :: gama,u1,u2,u3,p1,p2,p3,o1,o2,o3,cx,cy,cz
real, dimension(row_number,column_number), intent(out) :: v,vx,vy,vz
integer i,j,ip,im,jp,jm,row_numberm,row_numbermm,column_numberm,column_numbermm
real dx,dy,dz,dr,qu,qp,qo,gu,go,dr1,dr2,dr12,temp,drupwind,drdownwind

do j=1,column_number
    do i=1,row_number
        if (i .eq. 1) then
            if (velorder .eq. 2 .and. row_number .gt. 2) then
                ! routine for quadratic interpolation
                dx = cx(3,j)-cx(2,j)
                dy = cy(3,j)-cy(2,j)
                dz = cz(3,j)-cz(2,j)
                dr1 = sqrt(dx**2+dy**2+dz**2)
                dx = cx(2,j)-cx(1,j)
                dy = cy(2,j)-cy(1,j)
                dz = cz(2,j)-cz(1,j)
                dr2 = sqrt(dx**2+dy**2+dz**2)
                dr12 = dr1+dr2
                temp = (gama(1,j) - gama(3,j)) / dr12
                qu = dr12/dr2 * ((gama(2,j)-gama(3,j))/dr1 - temp) - temp
            else
                ! routine for linear interpolation
                dx = cx(2,j)-cx(1,j)
                dy = cy(2,j)-cy(1,j)
                dz = cz(2,j)-cz(1,j)
                dr = sqrt(dx**2+dy**2+dz**2)
                qu = (gama(2,j)-gama(1,j))/dr
            endif
        endif
        if (i .eq. row_number) then
            row_numberm = row_number-1
            if (velorder .eq. 2 .and. row_number .gt. 2) then
                row_numbermm = row_number-2
                ! routine for quadratic interpolation
                dx = cx(row_numbermm,j)-cx(row_numberm,j)
                dy = cy(row_numbermm,j)-cy(row_numberm,j)
                dz = cz(row_numbermm,j)-cz(row_numberm,j)
                dr1 = sqrt(dx**2+dy**2+dz**2)
                dx = cx(row_numberm,j)-cx(row_number,j)
                dy = cy(row_numberm,j)-cy(row_number,j)
                dz = cz(row_numberm,j)-cz(row_number,j)
                dr2 = sqrt(dx**2+dy**2+dz**2)
                dr12 = dr1+dr2
                temp = (gama(row_number,j) - gama(row_numbermm,j)) / dr12
                qu = temp - dr12/dr2 * ((gama(row_numberm,j)-gama(row_numbermm,j))/dr1 - temp)
            else
                ! routine for linear interpolation
                dx = cx(row_number,j)-cx(row_numberm,j)
                dy = cy(row_number,j)-cy(row_numberm,j)
                dz = cz(row_number,j)-cz(row_numberm,j)
                dr = sqrt(dx**2+dy**2+dz**2)
                qu = (gama(row_number,j)-gama(row_numberm,j))/dr
            endif
        endif
        if (j .eq. 1) then
            if (velorder .eq. 2 .and. column_number .gt. 2) then
                ! routine for quadratic interpolation
                dx = cx(i,3)-cx(i,2)
                dy = cy(i,3)-cy(i,2)
                dz = cz(i,3)-cz(i,2)
                dr1 = sqrt(dx**2+dy**2+dz**2)
                dx = cx(i,2)-cx(i,1)
                dy = cy(i,2)-cy(i,1)
                dz = cz(i,2)-cz(i,1)
                dr2 = sqrt(dx**2+dy**2+dz**2)
                dr12 = dr1+dr2
                temp = (gama(i,1) - gama(i,3)) / dr12
                qp = dr12/dr2 * ((gama(i,2)-gama(i,3))/dr1 - temp) - temp
            else
                ! routine for linear interpolation
                dx = cx(i,2)-cx(i,1)
                dy = cy(i,2)-cy(i,1)
                dz = cz(i,2)-cz(i,1)
                dr = sqrt(dx**2+dy**2+dz**2)
                qp = (gama(i,2)-gama(i,1))/dr
            endif
        endif
        if (j .eq. column_number) then
            column_numberm = column_number-1
            if (velorder .eq. 2 .and. column_number .gt. 2) then
                column_numbermm = column_number-2
                ! routine for quadratic interpolation
                dx = cx(i,column_numbermm)-cx(i,column_numberm)
                dy = cy(i,column_numbermm)-cy(i,column_numberm)
                dz = cz(i,column_numbermm)-cz(i,column_numberm)
                dr1 = sqrt(dx**2+dy**2+dz**2)
                dx = cx(i,column_numberm)-cx(i,column_number)
                dy = cy(i,column_numberm)-cy(i,column_number)
                dz = cz(i,column_numberm)-cz(i,column_number)
                dr2 = sqrt(dx**2+dy**2+dz**2)
                dr12 = dr1+dr2
                temp = (gama(i,column_number) - gama(i,column_numbermm)) / dr12
                qp = temp - dr12/dr2 * ((gama(i,column_numberm)-gama(i,column_numbermm))/dr1 - temp)
            else
                ! routine for linear interpolation
                dx = cx(i,column_number)-cx(i,column_numberm)
                dy = cy(i,column_number)-cy(i,column_numberm)
                dz = cz(i,column_number)-cz(i,column_numberm)
                dr = sqrt(dx**2+dy**2+dz**2)
                qp = (gama(i,column_number)-gama(i,column_numberm))/dr
            endif
        endif
        if (i .ne. 1 .and. i .ne. row_number) then
            ip = i+1
            im = i-1
            if (velorder .eq. 2) then
                ! routine for quadratic interpolation
                dx = cx(ip,j)-cx(i,j)
                dy = cy(ip,j)-cy(i,j)
                dz = cz(ip,j)-cz(i,j)
                drupwind = sqrt(dx**2+dy**2+dz**2)
                dx = cx(i,j)-cx(im,j)
                dy = cy(i,j)-cy(im,j)
                dz = cz(i,j)-cz(im,j)
                drdownwind = sqrt(dx**2+dy**2+dz**2)
                temp = (gama(ip,j)-gama(im,j)) / (drdownwind+drupwind)
                qu = temp - (drdownwind-drupwind) / drupwind *&
                    &((gama(i,j)-gama(im,j))/drdownwind - temp)
            elseif (velorder .eq. 1) then
                ! routine for linear interpolation
                dx = cx(ip,j)-cx(im,j)
                dy = cy(ip,j)-cy(im,j)
                dz = cz(ip,j)-cz(im,j)
                dr = sqrt(dx**2+dy**2+dz**2)
                qu = (gama(ip,j)-gama(im,j))/dr
            endif
        endif
        if (j .ne. 1 .and. j .ne. column_number) then
            jp = j+1
            jm = j-1
            if (velorder .eq. 2) then
                ! routine for quadratic interpolation
                dx = cx(i,jp)-cx(i,j)
                dy = cy(i,jp)-cy(i,j)
                dz = cz(i,jp)-cz(i,j)
                drupwind = sqrt(dx**2+dy**2+dz**2)
                dx = cx(i,j)-cx(i,jm)
                dy = cy(i,j)-cy(i,jm)
                dz = cz(i,j)-cz(i,jm)
                drdownwind = sqrt(dx**2+dy**2+dz**2)
                temp = (gama(i,jp)-gama(i,jm)) / (drdownwind+drupwind)
                qp = temp - (drdownwind-drupwind) / drupwind *&
                    &((gama(i,j)-gama(i,jm))/drdownwind - temp)
            elseif (velorder .eq. 1) then
                ! routine for linear interpolation
                dx = cx(i,jp)-cx(i,jm)
                dy = cy(i,jp)-cy(i,jm)
                dz = cz(i,jp)-cz(i,jm)
                dr = sqrt(dx**2+dy**2+dz**2)
                qp = (gama(i,jp)-gama(i,jm))/dr
            endif
        endif
        
        ! converting transversal to perpendicular induced speed component
        qo = (p1(i,j)*o1(i,j) + p2(i,j)*o2(i,j) + p3(i,j)*o3(i,j))*&
        &(qp - (u1(i,j)*p1(i,j) + u2(i,j)*p2(i,j) + u3(i,j)*p3(i,j))*qu)
        
        ! calculating speeds
        if (method .eq. 0) then
            ! calculating free stream components
            gu = u1(i,j)*speed_x + u2(i,j)*speed_y + u3(i,j)*speed_z
            go = o1(i,j)*speed_x + o2(i,j)*speed_y + o3(i,j)*speed_z
            
            vx(i,j) = (gu - qu)*u1(i,j) + (go - qo)*o1(i,j)
            vy(i,j) = (gu - qu)*u2(i,j) + (go - qo)*o2(i,j)
            vz(i,j) = (gu - qu)*u3(i,j) + (go - qo)*o3(i,j)
            
            v(i,j) = sqrt(vx(i,j)**2+vy(i,j)**2+vz(i,j)**2)
        elseif (method .eq. 1) then
            vx(i,j) = -qu*u1(i,j) - qo*o1(i,j)
            vy(i,j) = -qu*u2(i,j) - qo*o2(i,j)
            vz(i,j) = -qu*u3(i,j) - qo*o3(i,j)
            
            v(i,j) = sqrt(qu**2+qo**2)
        endif
    enddo
enddo

return
end subroutine

