! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  if not, see <http://www.gnu.org/licenses/>.

! calculate_velocities_surface.f90

subroutine calculate_velocities_surface(v,vx,vy,vz,method,velorder,&
    &gama,speed_x,speed_y,speed_z,column_number,row_number,&
    &u1,u2,u3,o1,o2,o3,cx,cy,cz)

implicit none

integer, intent(in) :: column_number,row_number,method,velorder
real, intent(in) :: speed_x,speed_y,speed_z
real, dimension(row_number,column_number), intent(in) :: gama,u1,u2,u3,o1,o2,o3,cx,cy,cz
real, dimension(row_number,column_number), intent(out) :: v,vx,vy,vz
integer i,j,ip,im,jp,jm,row_numberm,row_numbermm,column_numberm,column_numbermm
real gu,go
real, dimension(row_number,column_number) :: qu,qo

! central panels
if (row_number .gt. 2 .and. column_number .gt. 2) then
    do j=2,column_number-1
        do i=2,row_number-1
            ip = i+1
            im = i-1
            jp = j+1
            jm = j-1
            if (velorder .eq. 1) then
                ! routine for linear interpolation
                call speeds_surf_linear(qu(i,j),qo(i,j),&
                &cx(i,j),cy(i,j),cz(i,j),&
                &cx(im,jm),cx(ip,jm),cx(ip,jp),cx(im,jp),&
                &cy(im,jm),cy(ip,jm),cy(ip,jp),cy(im,jp),&
                &cz(im,jm),cz(ip,jm),cz(ip,jp),cz(im,jp),&
                &u1(i,j),u2(i,j),u3(i,j),&
                &o1(i,j),o2(i,j),o3(i,j),&
                &gama(im,jm),gama(ip,jm),gama(ip,jp),gama(im,jp))
            elseif (velorder .eq. 2) then
                ! routine for quadratic interpolation
                call speeds_surf_quadratic(qu(i,j),qo(i,j),&
                &cx(i,j),cy(i,j),cz(i,j),&
                &cx(im,j),cx(ip,j),cx(i,jm),cx(i,jp),&
                &cy(im,j),cy(ip,j),cy(i,jm),cy(i,jp),&
                &cz(im,j),cz(ip,j),cz(i,jm),cz(i,jp),&
                &u1(i,j),u2(i,j),u3(i,j),&
                &o1(i,j),o2(i,j),o3(i,j),&
                &gama(im,j),gama(ip,j),gama(i,jm),gama(i,jp),gama(i,j))
            endif
        enddo
    enddo
endif
! i=1 .and. i=row_number panels (j~=1 .and. j~=column_number)
if (column_number .gt. 2) then
    row_numberm = row_number-1
    row_numbermm = row_number-2
    do j=2,column_number-1
        jp = j+1
        jm = j-1
        if (velorder .eq. 1 .or. row_number .eq. 2) then  ! "row_number .eq. 2" for only two panel rows, only linear interpolation possible
            ! routine for linear interpolation
            call speeds_surf_linear(qu(1,j),qo(1,j),&
            &cx(1,j),cy(1,j),cz(1,j),&
            &cx(2,jm),cx(1,jm),cx(1,jp),cx(2,jp),&
            &cy(2,jm),cy(1,jm),cy(1,jp),cy(2,jp),&
            &cz(2,jm),cz(1,jm),cz(1,jp),cz(2,jp),&
            &u1(1,j),u2(1,j),u3(1,j),&
            &o1(1,j),o2(1,j),o3(1,j),&
            &gama(2,jm),gama(1,jm),gama(1,jp),gama(2,jp))
            call speeds_surf_linear(qu(row_number,j),qo(row_number,j),&
            &cx(row_number,j),cy(row_number,j),cz(row_number,j),&
            &cx(row_numberm,jm),cx(row_number,jm),cx(row_number,jp),cx(row_numberm,jp),&
            &cy(row_numberm,jm),cy(row_number,jm),cy(row_number,jp),cy(row_numberm,jp),&
            &cz(row_numberm,jm),cz(row_number,jm),cz(row_number,jp),cz(row_numberm,jp),&
            &u1(row_number,j),u2(row_number,j),u3(row_number,j),&
            &o1(row_number,j),o2(row_number,j),o3(row_number,j),&
            &gama(row_numberm,jm),gama(row_number,jm),gama(row_number,jp),gama(row_numberm,jp))
        elseif (velorder .eq. 2) then
            ! routine for quadratic interpolation
            call speeds_surf_quadratic(qu(1,j),qo(1,j),&
            &cx(1,j),cy(1,j),cz(1,j),&
            &cx(2,j),cx(3,j),cx(1,jm),cx(1,jp),&
            &cy(2,j),cy(3,j),cy(1,jm),cy(1,jp),&
            &cz(2,j),cz(3,j),cz(1,jm),cz(1,jp),&
            &u1(1,j),u2(1,j),u3(1,j),&
            &o1(1,j),o2(1,j),o3(1,j),&
            &gama(2,j),gama(3,j),gama(1,jm),gama(1,jp),gama(1,j))
            call speeds_surf_quadratic(qu(row_number,j),qo(row_number,j),&
            &cx(row_number,j),cy(row_number,j),cz(row_number,j),&
            &cx(row_numberm,j),cx(row_numbermm,j),cx(row_number,jm),cx(row_number,jp),&
            &cy(row_numberm,j),cy(row_numbermm,j),cy(row_number,jm),cy(row_number,jp),&
            &cz(row_numberm,j),cz(row_numbermm,j),cz(row_number,jm),cz(row_number,jp),&
            &u1(row_number,j),u2(row_number,j),u3(row_number,j),&
            &o1(row_number,j),o2(row_number,j),o3(row_number,j),&
            &gama(row_numberm,j),gama(row_numbermm,j),gama(row_number,jm),gama(row_number,jp),gama(row_number,j))
        endif
    enddo
endif
! j=1 .and. j=column_number panels (i~=1 .and. i~=row_number)
if (row_number .gt. 2) then
    column_numberm = column_number-1
    column_numbermm = column_number-2
    do i=2,row_number-1
        ip = i+1
        im = i-1
        if (velorder .eq. 1 .or. column_number .eq. 2) then  ! "column_number .eq. 2" for only two panel columns, only linear interpolation possible
            ! routine for linear interpolation
            call speeds_surf_linear(qu(i,1),qo(i,1),&
            &cx(i,1),cy(i,1),cz(i,1),&
            &cx(im,2),cx(im,1),cx(ip,1),cx(ip,2),&
            &cy(im,2),cy(im,1),cy(ip,1),cy(ip,2),&
            &cz(im,2),cz(im,1),cz(ip,1),cz(ip,2),&
            &u1(i,1),u2(i,1),u3(i,1),&
            &o1(i,1),o2(i,1),o3(i,1),&
            &gama(im,2),gama(im,1),gama(ip,1),gama(ip,2))
            call speeds_surf_linear(qu(i,column_number),qo(i,column_number),&
            &cx(i,column_number),cy(i,column_number),cz(i,column_number),&
            &cx(im,column_numberm),cx(im,column_number),cx(ip,column_number),cx(ip,column_numberm),&
            &cy(im,column_numberm),cy(im,column_number),cy(ip,column_number),cy(ip,column_numberm),&
            &cz(im,column_numberm),cz(im,column_number),cz(ip,column_number),cz(ip,column_numberm),&
            &u1(i,column_number),u2(i,column_number),u3(i,column_number),&
            &o1(i,column_number),o2(i,column_number),o3(i,column_number),&
            &gama(im,column_numberm),gama(im,column_number),gama(ip,column_number),gama(ip,column_numberm))
        elseif (velorder .eq. 2) then
            ! routine for quadratic interpolation
            call speeds_surf_quadratic(qu(i,1),qo(i,1),&
            &cx(i,1),cy(i,1),cz(i,1),&
            &cx(im,1),cx(ip,1),cx(i,2),cx(i,3),&
            &cy(im,1),cy(ip,1),cy(i,2),cy(i,3),&
            &cz(im,1),cz(ip,1),cz(i,2),cz(i,3),&
            &u1(i,1),u2(i,1),u3(i,1),&
            &o1(i,1),o2(i,1),o3(i,1),&
            &gama(im,1),gama(ip,1),gama(i,2),gama(i,3),gama(i,1))
            call speeds_surf_quadratic(qu(i,column_number),qo(i,column_number),&
            &cx(i,column_number),cy(i,column_number),cz(i,column_number),&
            &cx(im,column_number),cx(ip,column_number),cx(i,column_numberm),cx(i,column_numbermm),&
            &cy(im,column_number),cy(ip,column_number),cy(i,column_numberm),cy(i,column_numbermm),&
            &cz(im,column_number),cz(ip,column_number),cz(i,column_numberm),cz(i,column_numbermm),&
            &u1(i,column_number),u2(i,column_number),u3(i,column_number),&
            &o1(i,column_number),o2(i,column_number),o3(i,column_number),&
            &gama(im,column_number),gama(ip,column_number),gama(i,column_numberm),gama(i,column_numbermm),gama(i,column_number))
        endif
    enddo
endif
! corner panels
if (velorder .eq. 1 .or. row_number .eq. 2 .or. column_number .eq. 2) then
    ! routine for linear interpolation
    call speeds_surf_linear(qu(1,1),qo(1,1),&
    &cx(1,1),cy(1,1),cz(1,1),&
    &cx(1,1),cx(2,1),cx(1,2),cx(2,2),&
    &cy(1,1),cy(2,1),cy(1,2),cy(2,2),&
    &cz(1,1),cz(2,1),cz(1,2),cz(2,2),&
    &u1(1,1),u2(1,1),u3(1,1),&
    &o1(1,1),o2(1,1),o3(1,1),&
    &gama(1,1),gama(2,1),gama(1,2),gama(2,2))
    call speeds_surf_linear(qu(row_number,1),qo(row_number,1),&
    &cx(row_number,1),cy(row_number,1),cz(row_number,1),&
    &cx(row_number,1),cx(row_numberm,1),cx(row_number,2),cx(row_numberm,2),&
    &cy(row_number,1),cy(row_numberm,1),cy(row_number,2),cy(row_numberm,2),&
    &cz(row_number,1),cz(row_numberm,1),cz(row_number,2),cz(row_numberm,2),&
    &u1(row_number,1),u2(row_number,1),u3(row_number,1),&
    &o1(row_number,1),o2(row_number,1),o3(row_number,1),&
    &gama(row_number,1),gama(row_numberm,1),gama(row_number,2),gama(row_numberm,2))
    call speeds_surf_linear(qu(row_number,column_number),qo(row_number,column_number),&
    &cx(row_number,column_number),cy(row_number,column_number),cz(row_number,column_number),&
    &cx(row_number,column_number),cx(row_number,column_numberm),cx(row_numberm,column_number),cx(row_numberm,column_numberm),&
    &cy(row_number,column_number),cy(row_number,column_numberm),cy(row_numberm,column_number),cy(row_numberm,column_numberm),&
    &cz(row_number,column_number),cz(row_number,column_numberm),cz(row_numberm,column_number),cz(row_numberm,column_numberm),&
    &u1(row_number,column_number),u2(row_number,column_number),u3(row_number,column_number),&
    &o1(row_number,column_number),o2(row_number,column_number),o3(row_number,column_number),&
    &gama(row_number,column_number),gama(row_number,column_numberm),&
    &gama(row_numberm,column_number),gama(row_numberm,column_numberm))
    call speeds_surf_linear(qu(1,column_number),qo(1,column_number),&
    &cx(1,column_number),cy(1,column_number),cz(1,column_number),&
    &cx(1,column_number),cx(1,column_numberm),cx(2,column_number),cx(2,column_numberm),&
    &cy(1,column_number),cy(1,column_numberm),cy(2,column_number),cy(2,column_numberm),&
    &cz(1,column_number),cz(1,column_numberm),cz(2,column_number),cz(2,column_numberm),&
    &u1(1,column_number),u2(1,column_number),u3(1,column_number),&
    &o1(1,column_number),o2(1,column_number),o3(1,column_number),&
    &gama(1,column_number),gama(1,column_numberm),gama(2,column_number),gama(2,column_numberm))
elseif (velorder .eq. 2) then
    ! routine for quadratic interpolation
    call speeds_surf_quadratic(qu(1,1),qo(1,1),&
    &cx(1,1),cy(1,1),cz(1,1),&
    &cx(2,1),cx(3,1),cx(1,2),cx(1,3),&
    &cy(2,1),cy(3,1),cy(1,2),cy(1,3),&
    &cz(2,1),cz(3,1),cz(1,2),cz(1,3),&
    &u1(1,1),u2(1,1),u3(1,1),&
    &o1(1,1),o2(1,1),o3(1,1),&
    &gama(2,1),gama(3,1),gama(1,2),gama(1,3),gama(1,1))
    call speeds_surf_quadratic(qu(row_number,1),qo(row_number,1),&
    &cx(row_number,1),cy(row_number,1),cz(row_number,1),&
    &cx(row_number-1,1),cx(row_number-2,1),cx(row_number,2),cx(row_number,3),&
    &cy(row_number-1,1),cy(row_number-2,1),cy(row_number,2),cy(row_number,3),&
    &cz(row_number-1,1),cz(row_number-2,1),cz(row_number,2),cz(row_number,3),&
    &u1(row_number,1),u2(row_number,1),u3(row_number,1),&
    &o1(row_number,1),o2(row_number,1),o3(row_number,1),&
    &gama(row_number-1,1),gama(row_number-2,1),gama(row_number,2),gama(row_number,3),gama(row_number,1))
    call speeds_surf_quadratic(qu(row_number,column_number),qo(row_number,column_number),&
    &cx(row_number,column_number),cy(row_number,column_number),cz(row_number,column_number),&
    &cx(row_number-1,column_number),cx(row_number-2,column_number),cx(row_number,column_number-1),cx(row_number,column_number-2),&
    &cy(row_number-1,column_number),cy(row_number-2,column_number),cy(row_number,column_number-1),cy(row_number,column_number-2),&
    &cz(row_number-1,column_number),cz(row_number-2,column_number),cz(row_number,column_number-1),cz(row_number,column_number-2),&
    &u1(row_number,column_number),u2(row_number,column_number),u3(row_number,column_number),&
    &o1(row_number,column_number),o2(row_number,column_number),o3(row_number,column_number),&
    &gama(row_number-1,column_number),gama(row_number-2,column_number),gama(row_number,column_number-1),&
    &gama(row_number,column_number-2),gama(row_number,column_number))
    call speeds_surf_quadratic(qu(1,column_number),qo(1,column_number),&
    &cx(1,column_number),cy(1,column_number),cz(1,column_number),&
    &cx(1,column_number-1),cx(1,column_number-2),cx(2,column_number),cx(3,column_number),&
    &cy(1,column_number-1),cy(1,column_number-2),cy(2,column_number),cy(3,column_number),&
    &cz(1,column_number-1),cz(1,column_number-2),cz(2,column_number),cz(3,column_number),&
    &u1(1,column_number),u2(1,column_number),u3(1,column_number),&
    &o1(1,column_number),o2(1,column_number),o3(1,column_number),&
    &gama(1,column_number-1),gama(1,column_number-2),gama(2,column_number),gama(3,column_number),gama(1,column_number))
endif

! computing v,vx,vy,vz from locally induced velocities qu,qo
if (method .eq. 0) then          ! source/doublet combination velocity perturbations
    do j=1,column_number
        do i=1,row_number
            ! calculating free stream components
            gu = u1(i,j)*speed_x + u2(i,j)*speed_y + u3(i,j)*speed_z
            go = o1(i,j)*speed_x + o2(i,j)*speed_y + o3(i,j)*speed_z
            ! calculating velocity components (substituting from free stream components)
            vx(i,j) = (gu - qu(i,j))*u1(i,j) + (go - qo(i,j))*o1(i,j)
            vy(i,j) = (gu - qu(i,j))*u2(i,j) + (go - qo(i,j))*o2(i,j)
            vz(i,j) = (gu - qu(i,j))*u3(i,j) + (go - qo(i,j))*o3(i,j)
            ! calculating total velocity
            v(i,j) = sqrt(vx(i,j)**2+vy(i,j)**2+vz(i,j)**2)
        enddo
    enddo
elseif (method .eq. 1) then      ! doublets only total velocity
    do j=1,column_number
        do i=1,row_number
            ! calculating velocity components
            vx(i,j) = -qu(i,j)*u1(i,j) - qo(i,j)*o1(i,j)
            vy(i,j) = -qu(i,j)*u2(i,j) - qo(i,j)*o2(i,j)
            vz(i,j) = -qu(i,j)*u3(i,j) - qo(i,j)*o3(i,j)
            ! calculating total velocity
            v(i,j) = sqrt(qu(i,j)**2+qo(i,j)**2)
        enddo
    enddo
endif

return
end subroutine

