! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file comp_infl_const_doubl.f90

subroutine comp_infl_const_doubl(ii,jj,a,FF,error,S,&
    &column_number,row_number,row_number1,panel_number,&
    &u1,u2,u3,o1,o2,o3,n1,n2,n3,cx,cy,cz,x1,y1,x2,y2,x3,y3,x4,y4,d1,d2,d3,d4,&
    &column_number_other,row_number_other,row_number1_other,panel_number_other,&
    &cx_other,cy_other,cz_other)

implicit none

integer i,j,k,l,m,n
integer, intent(in) :: ii,jj,&
    &column_number,      row_number,      row_number1,      panel_number,&
    &column_number_other,row_number_other,row_number1_other,panel_number_other
real, intent(in) :: error
real, dimension(row_number1,column_number), intent(in) :: FF,S,&
    &u1,u2,u3,o1,o2,o3,n1,n2,n3,cx,cy,cz,x1,y1,x2,y2,x3,y3,x4,y4,d1,d2,d3,d4
real, dimension(row_number1_other,column_number_other), intent(in) :: cx_other,cy_other,cz_other
real, dimension(panel_number_other,panel_number), intent(out) :: a

! calculation of influence coefficients
m = 0
do k=1,row_number
    do l=1,column_number
        m = m+1
        n = 0
        do i=1,row_number_other
            do j=1,column_number_other
                n = n+1
                call infl_const_doubl(a(n,m),FF(k,l),&
                    &cx_other(i,j),cy_other(i,j),cz_other(i,j),cx(k,l),cy(k,l),cz(k,l),&
                    &u1(k,l),u2(k,l),u3(k,l),o1(k,l),o2(k,l),o3(k,l),n1(k,l),n2(k,l),n3(k,l),&
                    &S(k,l),x1(k,l),x2(k,l),x3(k,l),x4(k,l),y1(k,l),y2(k,l),y3(k,l),y4(k,l),&
                    &error,d1(k,l),d2(k,l),d3(k,l),d4(k,l))
                if (i .eq. k .and. j .eq. l .and. ii .eq. jj) then
                    a(n,m) = -0.5
                endif
            enddo
        enddo
    enddo
enddo

return
end subroutine

