! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file infl_const_doubl_sourc.f90

subroutine infl_const_doubl_sourc(a,b,FF,cx_other,cy_other,cz_other,cx,cy,cz,&
    &u1,u2,u3,o1,o2,o3,n1,n2,n3,S,x1,x2,x3,x4,y1,y2,y3,y4,error,d1,d2,d3,d4)

implicit none

real, intent(in) :: FF,cx_other,cy_other,cz_other,cx,cy,cz,&
    &u1,u2,u3,o1,o2,o3,n1,n2,n3,S,x1,x2,x3,x4,y1,y2,y3,y4,error,d1,d2,d3,d4
real, intent(out) :: a,b

real fourpi,FF_sqr,dist_x,dist_y,dist_z,cpx,cpy,cpz,rad,cpx1,cpx2,cpx3,cpx4,cpy1,cpy2,cpy3,cpy4,&
    &e1,e2,e3,e4,r1,r2,r3,r4,h1,h2,h3,h4,x21,x32,x43,x14,y21,y32,y43,y14,F,G,a1,a2,a3,a4,b1,b2,b3,b4

parameter (fourpi=0.079577471545948)

FF_sqr = FF**2
dist_x = cx_other-cx
dist_y = cy_other-cy
dist_z = cz_other-cz
cpx = dist_x*u1 + dist_y*u2 + dist_z*u3
cpy = dist_x*o1 + dist_y*o2 + dist_z*o3
cpz = dist_x*n1 + dist_y*n2 + dist_z*n3
rad = cpx**2 + cpy**2 + cpz**2
! if distance of panel from influenced point is greater
! then product of longer diagonal and "far field" coefficient
if (rad .gt. FF_sqr) then
    a = fourpi*S*cpz*rad**(-1.5)
    b = fourpi*S/sqrt(rad)
else
    cpx1 = cpx - x1
    cpx2 = cpx - x2
    cpx3 = cpx - x3
    cpx4 = cpx - x4
    cpy1 = cpy - y1
    cpy2 = cpy - y2
    cpy3 = cpy - y3
    cpy4 = cpy - y4
    e1 = cpx1**2+cpz**2
    e2 = cpx2**2+cpz**2
    e3 = cpx3**2+cpz**2
    e4 = cpx4**2+cpz**2
    r1 = sqrt(e1 + cpy1**2)
    r2 = sqrt(e2 + cpy2**2)
    r3 = sqrt(e3 + cpy3**2)
    r4 = sqrt(e4 + cpy4**2)
    x21 = x2-x1
    x32 = x3-x2
    x43 = x4-x3
    x14 = x1-x4
    y21 = y2-y1
    y32 = y3-y2
    y43 = y4-y3
    y14 = y1-y4
    if (abs(cpz) .lt. error) then
        if (d1 .lt. error) then
            b1 = 0
        else
            b1 = (cpx1*y21-cpy1*x21)/d1*log((r1+r2+d1)/(r1+r2-d1))
        endif
        if (d2 .lt. error) then
            b2 = 0
        else
            b2 = (cpx2*y32-cpy2*x32)/d2*log((r2+r3+d2)/(r2+r3-d2))
        endif
        if (d3 .lt. error) then
            b3 = 0
        else
            b3 = (cpx3*y43-cpy3*x43)/d3*log((r3+r4+d3)/(r3+r4-d3))
        endif
        if (d4 .lt. error) then
            b4 = 0
        else
            b4 = (cpx4*y14-cpy4*x14)/d4*log((r4+r1+d4)/(r4+r1-d4))
        endif
        a = 0
        b = -(b1+b2+b3+b4)*fourpi
    else
        h1 = cpx1*cpy1
        h2 = cpx2*cpy2
        h3 = cpx3*cpy3
        h4 = cpx4*cpy4
        if (d1 .lt. error) then
            a1 = 0
            b1 = 0
        else
            F = y21*e1 - x21*h1
            G = y21*e2 - x21*h2
            a1 = atan2(cpz*x21*(F*r2-G*r1), cpz**2*x21**2*r1*r2+F*G)
            b1 = (cpx1*y21-cpy1*x21)/d1*log((r1+r2+d1)/(r1+r2-d1))
        endif
        if (d2 .lt. error) then
            a2 = 0
            b2 = 0
        else
            F = y32*e2 - x32*h2
            G = y32*e3 - x32*h3
            a2 = atan2(cpz*x32*(F*r3-G*r2), cpz**2*x32**2*r2*r3+F*G)
            b2 = (cpx2*y32-cpy2*x32)/d2*log((r2+r3+d2)/(r2+r3-d2))
        endif
        if (d3 .lt. error) then
            a3 = 0
            b3 = 0
        else
            F = y43*e3 - x43*h3
            G = y43*e4 - x43*h4
            a3 = atan2(cpz*x43*(F*r4-G*r3), cpz**2*x43**2*r3*r4+F*G)
            b3 = (cpx3*y43-cpy3*x43)/d3*log((r3+r4+d3)/(r3+r4-d3))
        endif
        if (d4 .lt. error) then
            a4 = 0
            b4 = 0
        else
            F = y14*e4 - x14*h4
            G = y14*e1 - x14*h1
            a4 = atan2(cpz*x14*(F*r1-G*r4), cpz**2*x14**2*r4*r1+F*G)
            b4 = (cpx4*y14-cpy4*x14)/d4*log((r4+r1+d4)/(r4+r1-d4))
        endif
        a = -(a1+a2+a3+a4)*fourpi
        b = -(b1+b2+b3+b4)*fourpi-cpz*a
    endif
endif

return
end subroutine

