! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file read_key.f90

subroutine read_key(interactive,speed,ro,p_ref,mach,numalfa,line_alfa,numbeta,line_beta,wing_span,mac,wing_surf,origin,method,&
    &wake,error,farfield,collcalc,velorder,velometh,find_ac,res_req,res_ind,component_number,&
    &max_row,max_row1,max_row2,max_column,max_column1,key_err)

implicit none

integer, intent(in) :: interactive
integer, intent(out) :: numalfa,numbeta,collcalc,velorder,velometh,res_req,component_number,&
    &key_err,max_row,max_row1,max_row2,max_column,max_column1,method,find_ac
integer, dimension(15), intent(out) :: res_ind
real, intent(out) :: speed,ro,p_ref,mach,wing_span,mac,wing_surf,wake,error,farfield
real, dimension(3), intent(out) :: origin
character*100, intent(out) :: line_alfa,line_beta

integer i
integer, dimension(19) :: key_check
character*20, dimension(19) :: keyword_list
character*100 line
character*20 keyword,value
real temp_real,pi

parameter (pi=0.31415927E+01)
key_err = 0
key_check = (/0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0/)
keyword_list = (/"AIRSPEED","DENSITY","PRESSURE","MACH","ALFA","BETA","WINGSPAN","MAC",&
    &"SURFACE","FIND_AC","METHOD","WAKE","ERROR","FARFIELD","COLLCALC","VELORDER","VELOMETH","RESULTS","KOMP"/)

! reading starting APAME keyword
read (1,*) keyword
if (keyword .ne. 'APAME') then
    call message(interactive,"ERROR: Not an APAME input file")
    goto 999
endif
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! reading VERSION keyword
value = "0"
read (1,'(A)') line                         ! read line from the file
    read (line,*,end=501) keyword,value     ! read line from the memory
    goto 503
501 call message(interactive,"ERROR: VERSION sintax incorrect")
    goto 999
503   continue

if (keyword .ne. 'VERSION ') then
    call message(interactive,"ERROR: <VERSION> keyword should appear after <APAME input file> inside input file")
    goto 999
endif

! reading value for VERSION
read(value,*) temp_real
if (temp_real .ne. 2.2) then ! this value should be modified with current program version
    call message(interactive,"ERROR: Wrong input file version detected")
    goto 999
endif
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! READING KEYWORDS AND THEIR VALUES
do
    ! reading line
    read (1,'(A)',end=510) line
    if (line(1:1) .eq. '#') then        ! ignoring first character of the line
        goto 504
    elseif (line(1:2) .eq. '') then     ! ignoring line with first two characters blank
        goto 504
    else
        value = "0"     ! reseting value
        read (line,*,end=505) keyword,value
        goto 506
505     if (interactive .eq. 1) then
            write (6,213) "ERROR: keyword <",trim(keyword),"> does not have a value"
            write (2,213) "ERROR: keyword <",trim(keyword),"> does not have a value"
        else
            write (2,213) "ERROR: keyword <",trim(keyword),"> does not have a value"
        endif
        goto 999
506     continue
    endif
    if (keyword .eq. "AIRSPEED") then
        key_check(1) = 1
        read(value,*) speed
        if (speed .le. 0.) then
            call message(interactive,"ERROR: AIRSPEED parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "DENSITY") then
        key_check(2) = 1
        read(value,*) ro
        if (ro .le. 0.) then
            call message(interactive,"ERROR: DENSITY parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "PRESSURE") then
        key_check(3) = 1
        read(value,*) p_ref
        if (p_ref .le. 0.) then
            call message(interactive,"ERROR: PRESSURE parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "MACH") then
        key_check(4) = 1
        read(value,*) mach
        if (mach .lt. 0.) then
            call message(interactive,"ERROR: MACH parameter should be equal or greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "ALFA") then
        key_check(5) = 1
        read(value,*) numalfa
        if (numalfa .le. 0) then
            call message(interactive,"ERROR: ALFA parameter should be greater then zero")
            goto 999
        else
            read(1,'(A)') line_alfa
        endif
    elseif (keyword .eq. "BETA") then
        key_check(6) = 1
        read(value,*) numbeta
        if (numbeta .le. 0) then
            call message(interactive,"ERROR: BETA parameter should be greater then zero")
            goto 999
        else
            read(1,'(A)') line_beta
        endif
    elseif (keyword .eq. "WINGSPAN") then
        key_check(7) = 1
        read(value,*) wing_span
        if (wing_span .le. 0.) then
            call message(interactive,"ERROR: WINGSPAN parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "MAC") then
        key_check(8) = 1
        read(value,*) mac
        if (mac .le. 0.) then
            call message(interactive,"ERROR: MAC parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "SURFACE") then
        key_check(9) = 1
        read(value,*) wing_surf
        if (wing_surf .le. 0.) then
            call message(interactive,"ERROR: SURFACE parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "FIND_AC") then
        key_check(10) = 1
        read(value,*) find_ac
        if (find_ac .lt. 0 .or. find_ac .gt. 1) then
            call message(interactive,"ERROR: FIND_AC parameter should be either 0 or 1")
            goto 999
        else
            read(1,*) origin(1),origin(2),origin(3)
        endif
    elseif (keyword .eq. "METHOD") then
        key_check(11) = 1
        read(value,*) method
        if (method .lt. 0 .or. method .gt. 1) then
            call message(interactive,"ERROR: METHOD parameter should be either 0 or 1")
            goto 999
        endif
    elseif (keyword .eq. "WAKE") then
        key_check(12) = 1
        read(value,*) wake
        if (wake .le. 0.) then
            call message(interactive,"ERROR: WAKE parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "ERROR") then
        key_check(13) = 1
        read(value,*) error
        if (error .le. 0.) then
            call message(interactive,"ERROR: ERROR parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "FARFIELD") then
        key_check(14) = 1
        read(value,*) farfield
        if (farfield .le. 0.) then
            call message(interactive,"ERROR: FARFIELD parameter should be greater then zero")
            goto 999
        endif
    elseif (keyword .eq. "COLLCALC") then
        key_check(15) = 1
        read(value,*) collcalc
        if (collcalc .lt. 0 .or. collcalc .gt. 1) then
            call message(interactive,"ERROR: COLLCALC parameter should be either 0 or 1")
            goto 999
        endif
    elseif (keyword .eq. "VELORDER") then
        key_check(16) = 1
        read(value,*) velorder
        if (velorder .lt. 1 .or. velorder .gt. 2) then
            call message(interactive,"ERROR: VELORDER parameter should be either 1 or 2")
            goto 999
        endif
    elseif (keyword .eq. "VELOMETH") then
        key_check(17) = 1
        read(value,*) velometh
        if (velometh .lt. 0 .or. velometh .gt. 1) then
            call message(interactive,"ERROR: VELOMETH parameter should be either 0 or 1")
            goto 999
        endif
    elseif (keyword .eq. "RESULTS") then
        key_check(18) = 1
        read(value,*) res_req
        if (res_req .eq. 1 .or. res_req .eq. 0) then
            read (1,'(A)',end=600) line
            goto 601
600         call message(interactive,"ERROR: unexpected end of input file")
            goto 999
601         continue
            read(line,*,end=602) res_ind(1),res_ind(2),res_ind(3),res_ind(4),res_ind(5),res_ind(6),res_ind(7),res_ind(8),&
                &res_ind(9),res_ind(10),res_ind(11),res_ind(12),res_ind(13),res_ind(14),res_ind(15)
            goto 603
602         call message(interactive,"ERROR: inappropriate number of result requests")
            goto 999
603         read(line,*,end=604) res_ind(1),res_ind(2),res_ind(3),res_ind(4),res_ind(5),res_ind(6),res_ind(7),res_ind(8),&
                &res_ind(9),res_ind(10),res_ind(11),res_ind(12),res_ind(13),res_ind(14),res_ind(15),res_ind(15)
            goto 602
604         continue
            do i=1,8
                if (res_ind(i) .gt. 1 .or. res_ind(i) .lt. 0) then
                    call message(interactive,"ERROR: inappropriate values for result requests")
                    goto 999
                endif
            enddo
        else
            call message(interactive,"ERROR: RESULTS value should be either 1 or 0 for writing results file or not")
            goto 999
        endif
    elseif (keyword .eq. "KOMP") then
        key_check(19) = 1
        read(value,*) component_number
        if (component_number .le. 0) then
            call message(interactive,"ERROR: KOMP parameter should be greater then zero")
            goto 999
        else
            read(1,*) max_row1,max_column1
            max_row = max_row1-1
            max_row2 = max_row1+1
            max_column = max_column1-1
        endif
        goto 510
    else
        if (interactive .eq. 1) then
            write (6,213) "ERROR: Unknown keyword ",trim(keyword)," found"
            write (2,213) "ERROR: Unknown keyword ",trim(keyword)," found"
        else
            write (2,213) "ERROR: Unknown keyword ",trim(keyword)," found"
        endif
        goto 999
    endif
504 continue
enddo
510 continue

do i=1,12
    if (key_check(i) .eq. 0) then
        if (interactive .eq. 1) then
            write (6,213) "ERROR: Keyword ",trim(keyword_list(i))," missing"
            write (2,213) "ERROR: Keyword ",trim(keyword_list(i))," missing"
        else
            write (2,213) "ERROR: Keyword ",trim(keyword_list(i))," missing"
        endif
        goto 999
    endif
enddo

if (find_ac .eq. 1 .and. numalfa .eq. 1) then
    call message(interactive,"WARNING: Finding Aerodynamic Center can not be done with only one angle of attack...")
endif

213 format(1x,a,1x,a,1x,a)

goto 998
999 key_err = 1
998 continue

return
end subroutine

