! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file results.f90

subroutine results(interactive,file_name,speed,ro,p_ref,mach,&
    &numalfa,alfa,numbeta,beta,wing_span,mac,wing_surf,origin,method,wake,error,farfield,&
    &collcalc,velorder,velometh,find_ac,ac_count,&
    &component_number,component_name,lifting,case_number,x,y,z,cx,cy,cz,cp,v,vx,vy,vz,&
    &p_dyna,p_mano,p_stat,gama,sigm,S,FF,n1,n2,n3,u1,u2,u3,p1,p2,p3,o1,o2,o3,&
    &max_row,max_column,row_number,column_number,row_number1,column_number1,max_row1,max_row2,max_column1,&
    &coef_x,coef_y,coef_z,coef_l,coef_m,coef_n,&
    &Fx,Fy,Fz,Fl,Fm,Fn,&
    &Fx_comp,Fy_comp,Fz_comp,Fl_comp,Fm_comp,Fn_comp,&
    &coef_x_comp,coef_y_comp,coef_z_comp,coef_l_comp,coef_m_comp,coef_n_comp,res_ind)

implicit none

integer i,j,k,l,year,month,day,hour,minute
integer, intent(in) :: interactive,numalfa,numbeta,component_number,&
    &max_row,max_column,max_row1,max_row2,max_column1,case_number,collcalc,velorder,velometh,find_ac,ac_count,method
integer, intent(in), dimension(component_number) :: lifting,row_number,column_number,row_number1,column_number1
integer, intent(in), dimension(15) :: res_ind
character*100, intent(in) :: file_name
character*50, dimension(component_number), intent(in) :: component_name
real, intent(in) :: speed,ro,p_ref,mach,wing_span,mac,wing_surf,wake,error,farfield
real, dimension(3), intent(in) :: origin
real, dimension(numalfa), intent(in) :: alfa
real, dimension(numbeta), intent(in) :: beta
real, dimension(case_number), intent(in) :: coef_x,coef_y,coef_z,coef_l,coef_m,coef_n,Fx,Fy,Fz,Fl,Fm,Fn
real, dimension(component_number,case_number), intent(in) :: &
    &coef_x_comp,coef_y_comp,coef_z_comp,coef_l_comp,coef_m_comp,coef_n_comp,&
    &Fx_comp,Fy_comp,Fz_comp,Fl_comp,Fm_comp,Fn_comp
real, dimension(max_row2,max_column1,component_number), intent(in) :: x,y,z
real, dimension(max_row1,max_column,component_number), intent(in) :: &
    &S,FF,cx,cy,cz,n1,n2,n3,u1,u2,u3,p1,p2,p3,o1,o2,o3
real, dimension(max_row,max_column,component_number,case_number), intent(in) :: &
    &cp,v,vx,vy,vz,gama,sigm,p_dyna,p_mano,p_stat
character(8)  :: date
character(10) :: time

call message(interactive,"Start writing APAME results file...")

if (method .ne. 0) then
    call message(interactive," WARNING: SOURCE distribution not available for this method...")
endif

! open results file for writing
open(unit=3,file = trim(file_name)//'.res',status='replace')
call date_and_time(DATE=date,TIME=time)
read (date,100) year
read (date,101) month
read (date,102) day
read (time,103) hour
read (time,104) minute

write (3,206) "APAME results file"
write (3,106) day,month,year,hour,minute
write (3,206) "AIRSPEED"
write (3,207) speed
write (3,206) "DENSITY"
write (3,207) ro
write (3,206) "PRESSURE"
write (3,207) p_ref
write (3,206) "MACH"
write (3,207) mach
write (3,206) "ALFA"
write (3,208) numalfa
do i=1,numalfa
    write (3,209,advance="no") alfa(i)
enddo
write (3,105) ""
write (3,206) "BETA"
write (3,208) numbeta
do i=1,numbeta
    write (3,209,advance="no") beta(i)
enddo
write (3,105) ""
write (3,206) "WINGSPAN"
write (3,207) wing_span
write (3,206) "MAC"
write (3,207) mac
write (3,206) "SURFACE"
write (3,207) wing_surf
write (3,206) "FIND_AC"
write (3,208) find_ac
if (find_ac .eq. 1 .and. numalfa .gt. 1 .and. ac_count .eq. 10) then
    write (3,206) "WARNING: Finding Aerodynamic Center failed!"
    write (3,206) "Displaying original reference point..."
elseif (find_ac .eq. 1 .and. numalfa .gt. 1 .and. ac_count .lt. 10) then
    write (3,212) "Finding Aerodynamic Center successfull with",ac_count,"iterations!"
    write (3,206) "Displaying new reference point..."
elseif (find_ac .eq. 1 .and. numalfa .eq. 1) then
    write (3,206) "WARNING: Finding Aerodynamic Center can not be done with only one angle of attack!"
    write (3,206) "Displaying original reference point..."
endif
write (3,206) "ORIGIN"
do i=1,3
    write (3,209,advance="no") origin(i)
enddo
write (3,105) ""
write (3,206) "METHOD"
write (3,208) method
write (3,206) "WAKE"
write (3,207) wake
write (3,206) "ERROR"
write (3,207) error
write (3,206) "FARFIELD"
write (3,207) farfield
write (3,206) "COLLCALC"
write (3,208) collcalc
write (3,206) "VELORDER"
write (3,208) velorder
write (3,206) "VELOMETH"
write (3,208) velometh
write (3,206) "KOMP"
write (3,208) component_number
write (3,210) max_row,max_column

if (res_ind(1) .eq. 1) then
    write (3,206) "CX"
    do i=1,case_number
        write (3,209,advance="no") coef_x(i)
    enddo
    write (3,105) ""
    write (3,206) "CY"
    do i=1,case_number
        write (3,209,advance="no") coef_y(i)
    enddo
    write (3,105) ""
    write (3,206) "CZ"
    do i=1,case_number
        write (3,209,advance="no") coef_z(i)
    enddo
    write (3,105) ""
    write (3,206) "CL"
    do i=1,case_number
        write (3,209,advance="no") coef_l(i)
    enddo
    write (3,105) ""
    write (3,206) "CM"
    do i=1,case_number
        write (3,209,advance="no") coef_m(i)
    enddo
    write (3,105) ""
    write (3,206) "CN"
    do i=1,case_number
        write (3,209,advance="no") coef_n(i)
    enddo
    write (3,105) ""
endif

if (res_ind(2) .eq. 1) then
    write (3,206) "FX"
    do i=1,case_number
        write (3,209,advance="no") Fx(i)
    enddo
    write (3,105) ""
    write (3,206) "FY"
    do i=1,case_number
        write (3,209,advance="no") Fy(i)
    enddo
    write (3,105) ""
    write (3,206) "FZ"
    do i=1,case_number
        write (3,209,advance="no") Fz(i)
    enddo
    write (3,105) ""
    write (3,206) "FL"
    do i=1,case_number
        write (3,209,advance="no") Fl(i)
    enddo
    write (3,105) ""
    write (3,206) "FM"
    do i=1,case_number
        write (3,209,advance="no") Fm(i)
    enddo
    write (3,105) ""
    write (3,206) "FN"
    do i=1,case_number
        write (3,209,advance="no") Fn(i)
    enddo
    write (3,105) ""
endif

if (res_ind(3)+res_ind(4)+res_ind(5)+res_ind(6)+res_ind(7)+res_ind(8)+res_ind(9)+res_ind(10)&
        &+res_ind(11)+res_ind(12)+res_ind(13)+res_ind(14)+res_ind(15) .gt. 0) then
    do i=1,component_number
        write (3,211) "'",trim(component_name(i)),"'"
        write (3,208) lifting(i)
        write (3,210) row_number1(i),column_number1(i)

        if (res_ind(7) .eq. 1) then
            write (3,206) "X"
            do j=1,row_number1(i)
                do k=1,column_number1(i)
                    write (3,209,advance="no") x(j,k,i)
                enddo
                write (3,105) ""
            enddo

            write (3,206) "Y"
            do j=1,row_number1(i)
                do k=1,column_number1(i)
                    write (3,209,advance="no") y(j,k,i)
                enddo
                write (3,105) ""
            enddo

            write (3,206) "Z"
            do j=1,row_number1(i)
                do k=1,column_number1(i)
                    write (3,209,advance="no") z(j,k,i)
                enddo
                write (3,105) ""
            enddo
        endif

        if (res_ind(8) .eq. 1) then
            write (3,206) "COLX"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") cx(j,k,i)
                enddo
                write (3,105) ""
            enddo

            write (3,206) "COLY"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") cy(j,k,i)
                enddo
                write (3,105) ""
            enddo

            write (3,206) "COLZ"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") cz(j,k,i)
                enddo
                write (3,105) ""
            enddo
        endif

        if (res_ind(3) .eq. 1) then
            write (3,206) "CX_COMP"
            do j=1,case_number
                write (3,209,advance="no") coef_x_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "CY_COMP"
            do j=1,case_number
                write (3,209,advance="no") coef_y_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "CZ_COMP"
            do j=1,case_number
                write (3,209,advance="no") coef_z_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "CL_COMP"
            do j=1,case_number
                write (3,209,advance="no") coef_l_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "CM_COMP"
            do j=1,case_number
                write (3,209,advance="no") coef_m_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "CN_COMP"
            do j=1,case_number
                write (3,209,advance="no") coef_n_comp(i,j)
            enddo
            write (3,105) ""
        endif

        if (res_ind(4) .eq. 1) then
            write (3,206) "FX_COMP"
            do j=1,case_number
                write (3,209,advance="no") Fx_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "FY_COMP"
            do j=1,case_number
                write (3,209,advance="no") Fy_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "FZ_COMP"
            do j=1,case_number
                write (3,209,advance="no") Fz_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "FL_COMP"
            do j=1,case_number
                write (3,209,advance="no") Fl_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "FM_COMP"
            do j=1,case_number
                write (3,209,advance="no") Fm_comp(i,j)
            enddo
            write (3,105) ""

            write (3,206) "FN_COMP"
            do j=1,case_number
                write (3,209,advance="no") Fn_comp(i,j)
            enddo
            write (3,105) ""
        endif
        
        if (res_ind(12) .eq. 1) then
            write (3,206) "S"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") S(j,k,i)
                enddo
                write (3,105) ""
            enddo

            write (3,206) "FF"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") FF(j,k,i)
                enddo
                write (3,105) ""
            enddo

            write (3,206) "N1_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") n1(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "N2_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") n2(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "N3_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") n3(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "U1_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") u1(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "U2_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") u2(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "U3_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") u3(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "P1_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") p1(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "P2_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") p2(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "P3_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") p3(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "O1_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") o1(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "O2_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") o2(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
            write (3,206) "O3_VECTOR"
            do j=1,row_number(i)
                do k=1,column_number(i)
                    write (3,209,advance="no") o3(j,k,i)
                enddo
                write (3,105) ""
            enddo
            
        endif
        
        if (res_ind(5) .eq. 1) then
            write (3,206) "CP"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") cp(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
        endif

        if (res_ind(6) .eq. 1) then
            write (3,206) "V"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") v(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
        endif
        
        if (res_ind(9) .eq. 1) then
            write (3,206) "DIPOLE"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") gama(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
        endif
        
        if (res_ind(10) .eq. 1) then
            if (method .eq. 0) then
                write (3,206) "SOURCE"
                do k=1,case_number
                    write (3,208) k
                    do j=1,row_number(i)
                        do l=1,column_number(i)
                            write (3,209,advance="no") sigm(j,l,i,k)
                        enddo
                        write (3,105) ""
                    enddo
                enddo
            endif
        endif
        
        if (res_ind(11) .eq. 1) then
            write (3,206) "VX"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") vx(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
            write (3,206) "VY"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") vy(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
            write (3,206) "VZ"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") vz(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
        endif
        
        if (res_ind(13) .eq. 1) then
            write (3,206) "P_STAT"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") p_stat(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
        endif
        
        if (res_ind(14) .eq. 1) then
            write (3,206) "P_DYNA"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") p_dyna(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
        endif
        
        if (res_ind(15) .eq. 1) then
            write (3,206) "P_MANO"
            do k=1,case_number
                write (3,208) k
                do j=1,row_number(i)
                    do l=1,column_number(i)
                        write (3,209,advance="no") p_mano(j,l,i,k)
                    enddo
                    write (3,105) ""
                enddo
            enddo
        endif
        
    enddo
endif
write (3,206) "end"
close (3)

call message(interactive,"Done!")
call new_line(interactive)
call new_line(interactive)

100 format(i4)
101 format(4x,i2)
102 format(6x,i2)
103 format(i2)
104 format(2x,i2)
105 format(a)
106 format(1x,"Date:",1x,i2.2,"/",i2.2,"/",i4,2x,"Time:",1x,i2.2,":",i2.2)

206 format(1x,a)
207 format(1x,E14.8)
208 format(1x,i10)
209 format(1x,E14.8)
210 format(1x,i10,2x,i10)
211 format(1x,a,a,a)
212 format(1x,a,1x,i1,1x,a)

return
end subroutine

