! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the Gmax_rowU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file speeds.f90

subroutine speeds(case_number,numalfa,numbeta,speed,alfa,beta,Vx,Vy,Vz)

implicit none

integer i
integer, intent(in) :: case_number,numalfa,numbeta
real, intent(in) :: speed
real, dimension(numalfa), intent(in) :: alfa
real, dimension(numbeta), intent(in) :: beta
real, dimension(case_number), intent(out) :: Vx,Vy,Vz

if (numalfa .eq. 1 .and. numbeta .eq. 1) then
    Vx = speed*cos(alfa)*cos(beta)
    Vy = speed*sin(beta)
    Vz = speed*cos(beta)*sin(alfa)
elseif (numalfa .gt. 1 .and. numbeta .eq. 1) then
    do i=1,numalfa
        Vx(i) = speed*cos(alfa(i))
        Vy(i) = 0
        Vz(i) = speed*sin(alfa(i))
    enddo
elseif (numalfa .eq. 1 .and. numbeta .gt. 1) then
    do i=1,numbeta
        Vx(i) = speed*cos(alfa(1))*cos(beta(i))
        Vy(i) = speed*sin(beta(i))
        Vz(i) = speed*cos(beta(i))*sin(alfa(1))
    enddo
else
    do i=1,case_number
        if (i .gt. numalfa) then
            Vx(i) = speed*cos(beta(i-numalfa))
            Vy(i) = speed*sin(beta(i-numalfa))
            Vz(i) = 0
        else
            Vx(i) = speed*cos(alfa(i))
            Vy(i) = 0
            Vz(i) = speed*sin(alfa(i))
        endif
    enddo
endif

return
end subroutine

