! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file speeds_surf_linear.f90

subroutine speeds_surf_linear(qu,qo,&
    &x,y,z,&
    &x1,x2,x3,x4,&
    &y1,y2,y3,y4,&
    &z1,z2,z3,z4,&
    &u1,u2,u3,&
    &o1,o2,o3,&
    &gama1,gama2,gama3,gama4)

implicit none

real, intent(in) :: x,y,z,x1,x2,x3,x4,y1,y2,y3,y4,z1,z2,z3,z4,u1,u2,u3,o1,o2,o3,gama1,gama2,gama3,gama4
real, intent(out) :: qu,qo
integer info,lwork

real, dimension(6) :: work
real, dimension(4,3) :: a
real, dimension(4,1) :: gama_vector

parameter (lwork=6)


a(1,1) = (x1-x)*u1+(y1-y)*u2+(z1-z)*u3
a(1,2) = (x1-x)*o1+(y1-y)*o2+(z1-z)*o3
a(1,3) = 1.

a(2,1) = (x2-x)*u1+(y2-y)*u2+(z2-z)*u3
a(2,2) = (x2-x)*o1+(y2-y)*o2+(z2-z)*o3
a(2,3) = 1.

a(3,1) = (x3-x)*u1+(y3-y)*u2+(z3-z)*u3
a(3,2) = (x3-x)*o1+(y3-y)*o2+(z3-z)*o3
a(3,3) = 1.

a(4,1) = (x4-x)*u1+(y4-y)*u2+(z4-z)*u3
a(4,2) = (x4-x)*o1+(y4-y)*o2+(z4-z)*o3
a(4,3) = 1.

gama_vector(1,1) = gama1
gama_vector(2,1) = gama2
gama_vector(3,1) = gama3
gama_vector(4,1) = gama4

call sgels('N', 4, 3, 1, a, 4, gama_vector, 4, work, lwork, info)

qu = gama_vector(1,1)
qo = gama_vector(2,1)

return
end subroutine

