! Copyright (C) 2009  Daniel Filkovic

! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
! %                                                      %
! %            APAME - Aircraft Panel Method             %
! %______________________________________________________%
! %                                                      %
! %              3D potential flow solver                %
! %                                                      %
! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

! This file is part of APAME.

! APAME is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or
! (at your option) any later version.

! APAME is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with APAME.  If not, see <http://www.gnu.org/licenses/>.

! file speeds_surf_quadratic.f90

subroutine speeds_surf_quadratic(qu,qo,&
    &x,y,z,&
    &x1,x2,x3,x4,&
    &y1,y2,y3,y4,&
    &z1,z2,z3,z4,&
    &u1,u2,u3,&
    &o1,o2,o3,&
    &gama1,gama2,gama3,gama4,gama5)

implicit none

real, intent(in) :: x,y,z,x1,x2,x3,x4,y1,y2,y3,y4,z1,z2,z3,z4,u1,u2,u3,o1,o2,o3,gama1,gama2,gama3,gama4,gama5
real, intent(out) :: qu,qo
integer info
real x1l,x2l,x3l,x4l,y1l,y2l,y3l,y4l
real, dimension(5,5) :: T
integer, dimension(5,5) :: ipiv
real, dimension(5,1) :: gama_vector


x1l = (x1-x)*u1+(y1-y)*u2+(z1-z)*u3
y1l = (x1-x)*o1+(y1-y)*o2+(z1-z)*o3

x2l = (x2-x)*u1+(y2-y)*u2+(z2-z)*u3
y2l = (x2-x)*o1+(y2-y)*o2+(z2-z)*o3

x3l = (x3-x)*u1+(y3-y)*u2+(z3-z)*u3
y3l = (x3-x)*o1+(y3-y)*o2+(z3-z)*o3

x4l = (x4-x)*u1+(y4-y)*u2+(z4-z)*u3
y4l = (x4-x)*o1+(y4-y)*o2+(z4-z)*o3

T(1,1) = x1l**2
T(1,2) = y1l**2
T(1,3) = x1l
T(1,4) = y1l
T(1,5) = 1.

T(2,1) = x2l**2
T(2,2) = y2l**2
T(2,3) = x2l
T(2,4) = y2l
T(2,5) = 1.

T(3,1) = x3l**2
T(3,2) = y3l**2
T(3,3) = x3l
T(3,4) = y3l
T(3,5) = 1.

T(4,1) = x4l**2
T(4,2) = y4l**2
T(4,3) = x4l
T(4,4) = y4l
T(4,5) = 1.

T(5,1) = 0.
T(5,2) = 0.
T(5,3) = 0.
T(5,4) = 0.
T(5,5) = 1.

gama_vector(1,1) = gama1
gama_vector(2,1) = gama2
gama_vector(3,1) = gama3
gama_vector(4,1) = gama4
gama_vector(5,1) = gama5

call sgesv(5,1,T,5,ipiv,gama_vector,5,info)

qu = gama_vector(3,1)
qo = gama_vector(4,1)

return
end subroutine

